/**********************************************************************
 ** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
 **
 ** This file is part of the Qtopia Environment.
 **
 ** This file may be distributed and/or modified under the terms of the
 ** GNU General Public License version 2 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.
 **
 ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 **
 ** See http://www.trolltech.com/gpl/ for GPL licensing information.
 **
 ** Contact info@trolltech.com if any conditions of this licensing are
 ** not clear to you.
 **
 **********************************************************************/

#include "sysdep/qualendardb.h"
#include "sysdep/qualendarcategory.h"
#include "evententry.h"
#include "repeatentry.h"
#include "utils/fileselector.h"
#include "qualendarglobal.h"

#include <qaction.h>
#include <qcheckbox.h>
#include <qfontmetrics.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qscrollview.h>
#include <qspinbox.h>
#include <qtoolbutton.h>

#include <qpe/qpeapplication.h>
#include <qpe/global.h>
#include <qpe/timeconversion.h>
#include <qpe/timestring.h>
#include <qpe/tzselect.h>
#include <qpe/config.h>
#include <qpe/qpemenubar.h>
#include <qpe/resource.h>

#include "utils/qualendarcategorycombo.h"
#include "mymultilineedit.h"

#include <stdlib.h>

#define N_DESC 10
#define N_LOC 10

/*
 *  Constructs a EventEntry which is a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */

EventEntry::EventEntry( bool startOnMonday, const QDateTime &start,
			const QDateTime &end, bool whichClock, QWidget* parent,
			const char* name )
  : EventEntryBase( parent, name ),
    ampm( whichClock ),
    startWeekOnMonday( startOnMonday ) 
{
  init();
  setDates(start, end);
}

static void addOrPick( QComboBox* combo, const QString& t )
{
  for (int i=0; i<combo->count(); i++) {
    if ( combo->text(i) == t ) {
      combo->setCurrentItem(i);
      return;
    }
  }
  combo->setEditText(t);
}

void EventEntry::setEvent(const Event &event) {
  e = event;
  setDates(event.start(), event.end()); 

  QString catName;
  if (event.categories().count() == 0)
    catName = QObject::tr("Unfiled");
  else
    catName = QualendarGlobal::categories.label("Calendar", 
						event.categories()[0]);
  comboCategory->setCurrent(catName);
	
  if(event.description().isEmpty())
    comboDescription->lineEdit()->clear();
  else
    addOrPick( comboDescription, event.description() );
  if(event.location().isEmpty())
    comboLocation->lineEdit()->clear();
  else
    addOrPick( comboLocation, event.location() );
  comboDescription->lineEdit()->home(false);
  comboLocation->lineEdit()->home(false);
  checkAlarm->setChecked( event.hasAlarm() );
  checkAllDay->setChecked( event.type() == Event::AllDay );
  if(event.notes().isEmpty()) 
    editNote->clear();
  else 
    editNote->setText(event.notes());
  spinAlarm->setValue(event.alarmTime());
  if ( event.alarmSound() == Event::Silent )
    comboSound->setCurrentItem( 0 );
  else
    comboSound->setCurrentItem( 1 );
  if ( event.hasRepeat() ) {
    rp = event.repeatPattern();
    //	cmdRepeat->setText( tr("Repeat...") );
  } else {
    Event::RepeatPattern rp0;
    rp0.type = Event::NoRepeat;
    rp = rp0;
    //	cmdRepeat->setText( tr("No Repeat...") );
  }
  setRepeatLabel();
  comboDescription->lineEdit()->setFocus();
  comboDescription->lineEdit()->end(false);
}
EventEntry::EventEntry( bool startOnMonday, const Event &event, bool whichClock,
			QWidget* parent,  const char* name )
  : EventEntryBase( parent, name ),
    ampm( whichClock ),
    startWeekOnMonday( startOnMonday )
{
  init();
  setEvent(event);
}

void EventEntry::setDates( const QDateTime& s, const QDateTime& e )
{
  int shour,
    ehour;
  QString strStart,
    strEnd;
  startDate = s.date();
  endDate = e.date();
  startTime = s.time();
  endTime = e.time();
  startDateChanged(s.date());
  if ( ampm ) {
    shour = s.time().hour();
    ehour = e.time().hour();
    if ( shour >= 12 ) {
      if ( shour > 12 )
	shour -= 12;
      strStart.sprintf( "%d:%02d PM", shour, s.time().minute() );
    } else {
      if ( shour == 0 )
	shour = 12;
      strStart.sprintf( "%d:%02d AM", shour, s.time().minute() );
    }
    if ( ehour == 24 && e.time().minute() == 0 ) {
      strEnd  = "11:59 PM"; // or "midnight"
    } else if ( ehour >= 12 ) {
      if ( ehour > 12 )
	ehour -= 12;
      strEnd.sprintf( "%d:%02d PM", ehour, e.time().minute() );
    } else {
      if ( ehour == 0 )
	ehour = 12;
      strEnd.sprintf( "%d:%02d AM", ehour, e.time().minute() );
    }
  } else {
    strStart.sprintf( "%02d:%02d", s.time().hour(), s.time().minute() );
    strEnd.sprintf( "%02d:%02d", e.time().hour(), e.time().minute() );
  }
  addOrPick(comboStart, strStart );
  endDateChanged(e.date());
  addOrPick(comboEnd, strEnd );
}

void EventEntry::init()
{
  QPopupMenu *editMenu = new QPopupMenu(this);
  MenuBar->insertItem(tr("Edit"), editMenu);
  MenuBar->setFixedWidth(120); // FIXME
  QAction *a;
  a = new QAction(tr("Attach file"), Resource::loadPixmap("open"),
		  QString::null, 0, this, 0);
  a->addTo(editMenu);
  connect(a, SIGNAL(activated()), this, SLOT(slotFile()));

  comboDescription->setInsertionPolicy(QComboBox::AtCurrent);
  comboLocation->setInsertionPolicy(QComboBox::AtCurrent);

  QWidget *disp = QApplication::desktop();
  // dirty hack
  bool bigDisplay = (disp->width() * disp->height() >= 100000); 
  if (bigDisplay) {
    ToolButtonCut->setFixedSize(32, 32);
    ToolButtonCopy->setFixedSize(32, 32);
    ToolButtonPaste->setFixedSize(32, 32);
    ToolButtonFile->setFixedSize(32, 32);
  }

  {
    comboDescription->clear();
    comboLocation->clear();
    Config config("QualendarTmp");
    config.setGroup("Main");
    for(int i=0; i<N_DESC; i++) {
      QString s;
      s.append("DescStr");
      s.append(QString::number(i));
      comboDescription->insertItem(config.readEntry(s));
      comboDescription->setEditText("");
    }
    for(int i=0; i<N_DESC; i++) {
      QString s;
      s.append("LocStr");
      s.append(QString::number(i));
      comboLocation->insertItem(config.readEntry(s));
      comboLocation->setEditText("");
    }
  }

  initCombos();
  connect(buttonStart, SIGNAL(clicked()), 
  	  this, SLOT(slotStart()));
  buttonStart->setAutoDefault(false);
  connect(buttonEnd, SIGNAL(clicked()), 
	  this, SLOT(slotEnd()));
  buttonEnd->setAutoDefault(false);
  connect( qApp, SIGNAL( clockChanged( bool ) ),
	   this, SLOT( slotChangeClock( bool ) ) );
  connect( qApp, SIGNAL(weekChanged(bool)),
	   this, SLOT(slotChangeStartOfWeek(bool)) );

  comboDescription->setFocus();

  connect(ToolButtonCut, SIGNAL(clicked()), this, SLOT(slotCut()));
  connect(ToolButtonCopy, SIGNAL(clicked()), this, SLOT(slotCopy()));
  connect(ToolButtonPaste, SIGNAL(clicked()), this, SLOT(slotPaste()));
  connect(ToolButtonFile, SIGNAL(clicked()), this, SLOT(slotFile()));
}

/*
 *  Destroys the object and frees any allocated resources
 */
EventEntry::~EventEntry()
{
  // no need to delete child widgets, Qt does it all for us
}

/*
 * public slot
 */
void EventEntry::endDateChanged(QDate date)
{
  endDate = date;
  if ( endDate < startDate ) {
    endDate = startDate;
  }

  buttonEnd->setText( TimeString::shortDate( endDate ) );
}

static QTime parseTime( const QString& s, bool ampm )
{
  QTime tmpTime;
  QStringList l = QStringList::split( ':', s );
  int hour = l[0].toInt();
  if ( ampm ) {
    int i=0;
    while (i<int(l[1].length()) && l[1][i]>='0' && l[1][i]<='9')
      i++;
    QString digits = l[1].left(i);
    if ( l[1].contains( "PM", FALSE ) ) {
      if ( hour != 12 )
	hour += 12;
    } else {
      if ( hour == 12 )
	hour = 0;
    }
    l[1] = digits;
  }
  int minute = l[1].toInt();
  if ( minute > 59 )
    minute = 59;
  else if ( minute < 0 )
    minute = 0;
  if ( hour > 23 ) {
    hour = 23;
    minute = 59;
  } else if ( hour < 0 )
    hour = 0;
  tmpTime.setHMS( hour, minute, 0 );
  return tmpTime;
}

/*
 * public slot
 */
void EventEntry::endTimeChanged( const QString &s )
{
  QTime tmpTime = parseTime(s,ampm);
  if ( endDate > startDate || tmpTime >= startTime ) {
    endTime = tmpTime;
  } else {
    endTime = startTime;
    comboEnd->setCurrentItem( comboStart->currentItem() );
  }
}

/*
 * public slot
 */
void EventEntry::startDateChanged(QDate date)
{
  QDate prev = startDate;
  startDate = date;
  if ( rp.type == Event::Weekly &&
       startDate.dayOfWeek() != prev.dayOfWeek() ) {
    // if we change the start of a weekly repeating event
    // set the repeating day appropriately
    char mask = 1 << (prev.dayOfWeek()-1);
    rp.days &= (~mask);
    rp.days |= 1 << (startDate.dayOfWeek()-1);
  }

  buttonStart->setText( TimeString::shortDate( startDate ) );
}

/*
 * public slot
 */
void EventEntry::startTimeChanged( int index )
{
  startTime = parseTime(comboStart->text(index),ampm);
  changeEndCombo( index );
}
/*
 * public slot
 */
void EventEntry::typeChanged( const QString &s )
{
  bool b = s != "All Day";
  buttonStart->setEnabled( b );
  comboStart->setEnabled( b );
  comboEnd->setEnabled( b );
}
/*
 * public slot
 */
void EventEntry::changeEndCombo( int change )
{
  if ( change + 2 < comboEnd->count() )
    change += 2;
  comboEnd->setCurrentItem( change );
  endTimeChanged( comboEnd->currentText() );
}

void EventEntry::slotRepeat()
{
  // Work around for compiler Bug..
  RepeatEntry *e;

  // it is better in my opinion to just grab this from the mother,
  // since, this dialog doesn't need to keep track of it...
  if ( rp.type != Event::NoRepeat )
    e = new RepeatEntry( startWeekOnMonday, rp, startDate, this);
  else
    e = new RepeatEntry( startWeekOnMonday, startDate, this );

#if defined(Q_WS_QWS) || defined(_WS_QWS_)
  e->showMaximized();
#endif
  if ( e->exec() ) {
    rp = e->repeatPattern();
    setRepeatLabel();
  }
}

void EventEntry::slotChangeStartOfWeek( bool onMonday )
{
  startWeekOnMonday = onMonday;
}

Event EventEntry::event()
{
  Event ev;
  Event::SoundTypeChoice st;
  ev.setDescription( comboDescription->currentText() );
  ev.setLocation( comboLocation->currentText() );
  if (comboCategory->currentText() != tr("Unfiled"))
    ev.setCategories(QualendarGlobal::categories.id("Calendar", comboCategory->currentText()));
  ev.setType( checkAllDay->isChecked() ? Event::AllDay : Event::Normal );
  if ( startDate > endDate ) {
    QDate tmp = endDate;
    endDate = startDate;
    startDate = tmp;
  }
  startTime = parseTime( comboStart->currentText(), ampm );
  endTime = parseTime( comboEnd->currentText(), ampm );
  if ( startTime > endTime && endDate == startDate ) {
    QTime tmp = endTime;
    endTime = startTime;
    startTime = tmp;
  }
  // don't set the time if theres no need too
  if ( ev.type() == Event::AllDay ) {
    startTime.setHMS( 0, 0, 0 );
    endTime.setHMS( 23, 59, 59 );
  }

  // adjust start and end times based on timezone
  QDateTime start( startDate, startTime );
  QDateTime end( endDate, endTime );
  time_t start_utc, end_utc;

  //    qDebug( "tz: %s", timezone->currentZone().latin1() );

  // get real timezone
  QString realTZ;
  realTZ = QString::fromLocal8Bit( getenv("TZ") );

  // set timezone
  if ( setenv( "TZ", timezone->currentZone(), true ) != 0 )
    qWarning( "There was a problem setting the timezone." );

  // convert to UTC based on selected TZ (calling tzset internally)
  start_utc = TimeConversion::toUTC( start );
  end_utc = TimeConversion::toUTC( end );

  // done playing around... put it all back
  unsetenv( "TZ" );
  if ( !realTZ.isNull() )
    if ( setenv( "TZ", realTZ, true ) != 0 )
      qWarning( "There was a problem setting the timezone." );

  // convert UTC to local time (calling tzset internally)
  ev.setStart( TimeConversion::fromUTC( start_utc ) );
  ev.setEnd( TimeConversion::fromUTC( end_utc ) );

  // we only have one type of sound at the moment... LOUD!!!
  if ( comboSound->currentItem() != 0 )
    st = Event::Loud;
  else
    st = Event::Silent;
  ev.setAlarm( checkAlarm->isChecked(), spinAlarm->value(), st );
  if ( rp.type != Event::NoRepeat )
    ev.setRepeat( TRUE, rp );
  ev.setNotes( editNote->text() );


  {
    Config config("QualendarTmp");
    config.setGroup("Main");
    int i;
    for (i=0; i<N_DESC; i++) {
      if ( comboDescription->text(i) == comboDescription->currentText() )
	break;
    }
    if (i == N_DESC && comboDescription->currentText().length() > 0) {
      comboDescription->insertItem(comboDescription->currentText(), 0);
    }

    for(i=0; i<N_DESC; i++) {
      QString s;
      s.append("DescStr");
      s.append(QString::number(i));
      config.writeEntry(s, comboDescription->text(i));
    }

    for (i=0; i<N_DESC; i++) {
      if ( comboLocation->text(i) == comboLocation->currentText() )
	break;
    }
    if (i == N_DESC && comboLocation->currentText().length() > 0) {
      comboLocation->insertItem(comboLocation->currentText(), 0);
    }
    for(i=0; i<N_DESC; i++) {
      QString s;
      s.append("LocStr");
      s.append(QString::number(i));
      config.writeEntry(s, comboLocation->text(i));
    }
  }



  return ev;
}

void EventEntry::setRepeatLabel()
{

  switch( rp.type ) {
  case Event::Daily:
    cmdRepeat->setText( tr("Daily...") );
    break;
  case Event::Weekly:
    cmdRepeat->setText( tr("Weekly...") );
    break;
  case Event::MonthlyDay:
  case Event::MonthlyDate:
    cmdRepeat->setText( tr("Monthly...") );
    break;
  case Event::Yearly:
    cmdRepeat->setText( tr("Yearly...") );
    break;
  default:
    cmdRepeat->setText( tr("No Repeat...") );
  }
}

void EventEntry::setAlarmEnabled( bool alarmPreset, int presetTime, Event::SoundTypeChoice sound )
{
  checkAlarm->setChecked( alarmPreset );
  spinAlarm->setValue( presetTime );
  if ( sound != Event::Silent )
    comboSound->setCurrentItem( 1 );
  else
    comboSound->setCurrentItem( 0 );
}

void EventEntry::setWidth(int w) 
{
  QFontMetrics fm(TextLabel1->font());
  comboDescription->setMaximumWidth(w - fm.width(TextLabel1->text()) - 20);
  comboLocation->setMaximumWidth(w - fm.width(TextLabel2->text()) - 20);
}

void EventEntry::initCombos()
{
  comboStart->clear();
  comboEnd->clear();
  if ( ampm ) {
    for ( int i = 0; i < 24; i++ ) {
      if ( i == 0 ) {
	comboStart->insertItem( "12:00 AM" );
	comboStart->insertItem( "12:30 AM" );
	comboEnd->insertItem( "12:00 AM" );
	comboEnd->insertItem( "12:30 AM" );
      } else if ( i == 12 ) {
	comboStart->insertItem( "12:00 PM" );
	comboStart->insertItem( "12:30 PM" );
	comboEnd->insertItem( "12:00 PM" );
	comboEnd->insertItem( "12:30 PM" );
      } else if ( i > 12 ) {
	comboStart->insertItem( QString::number( i - 12 ) + ":00 PM" );
	comboStart->insertItem( QString::number( i - 12 ) + ":30 PM" );
	comboEnd->insertItem( QString::number( i - 12 ) + ":00 PM" );
	comboEnd->insertItem( QString::number( i - 12 ) + ":30 PM" );
      } else {
	comboStart->insertItem( QString::number( i) + ":00 AM" );
	comboStart->insertItem( QString::number( i ) + ":30 AM" );
	comboEnd->insertItem( QString::number( i ) + ":00 AM" );
	comboEnd->insertItem( QString::number( i ) + ":30 AM" );
      }
    }
  } else {
    for ( int i = 0; i < 24; i++ ) {
      if ( i < 10 ) {
	comboStart->insertItem( QString("0")
				+ QString::number(i) + ":00" );
	comboStart->insertItem( QString("0")
				+ QString::number(i) + ":30" );
	comboEnd->insertItem( QString("0")
			      + QString::number(i) + ":00" );
	comboEnd->insertItem( QString("0")
			      + QString::number(i) + ":30" );
      } else {
	comboStart->insertItem( QString::number(i) + ":00" );
	comboStart->insertItem( QString::number(i) + ":30" );
	comboEnd->insertItem( QString::number(i) + ":00" );
	comboEnd->insertItem( QString::number(i) + ":30" );
      }
    }
  }
}

void EventEntry::slotChangeClock( bool whichClock )
{
  ampm = whichClock;
  initCombos();
  setDates( QDateTime( startDate, startTime ), QDateTime( endDate, endTime ) );
}

void EventEntry::slotCut()
{
  QWidget *w = focusWidget();
  if (w->inherits("QLineEdit"))
    ((QLineEdit*)w)->cut();
  if (w->inherits("QMultiLineEdit"))
    ((QMultiLineEdit*)w)->cut();
}
void EventEntry::slotCopy()
{
  QWidget *w = focusWidget();
  if (w->inherits("QLineEdit"))
    ((QLineEdit*)w)->copy();
  if (w->inherits("QMultiLineEdit"))
    ((QMultiLineEdit*)w)->copy();
}
void EventEntry::slotPaste()
{
  QWidget *w = focusWidget();
  if (w->inherits("QLineEdit"))
    ((QLineEdit*)w)->paste();
  if (w->inherits("QMultiLineEdit"))
    ((QMultiLineEdit*)w)->paste();
}
void EventEntry::slotFile()
{
  QString dir;
  {
    Config config("QualendarTmp");
    config.setGroup("Main");
    dir = config.readEntry("CurrentDir", 
			   QPEApplication::documentDir().latin1());
  }
  FileSelector dlg(QDate::currentDate(), dir,
		   this, "file", true);
  dlg.showMaximized();
  if (dlg.exec() == QDialog::Accepted) {
    QString s = dlg.absName();
    editNote->insert("[[file://" + s + "]]\n");

  }
  Config config("QualendarTmp");
  config.setGroup("Main");
  config.writeEntry("CurrentDir", dlg.currentDir());
}

void EventEntry::slotStart() {
  DatePickerDialog dlg(this, startDate);
  dlg.exec();
  startDateChanged(dlg.date());
  endDateChanged(dlg.date());
}
void EventEntry::slotEnd() {
  DatePickerDialog dlg(this, endDate);
  dlg.exec();
  endDateChanged(dlg.date());
}
