#ifndef QUALENDARVIEW_H
#define QUALENDARVIEW_H

#include <qarray.h>
#include <qwidgetstack.h>
#include <qvbox.h>
#include <qmap.h>
#include <qbitarray.h>
#include <qvaluelist.h>
#include <qdatetime.h>
#include <qpixmap.h>
#include <qpe/event.h>
#include "sysdep/qualendarsysdep.h"

#include "qualendaritem.h"

class QPopupMenu;
class QDateTime;
class QualendarView;

/**
 * The widget containing various views.
 * 
 * This widget is used as the central widget in the main window
 * and contains various view widgets. 
 * 
 * For signals viewItem etc., see documentation for QualendarView.
 **/
class QualendarViewStack : public QWidgetStack {
  Q_OBJECT
 public:
  /**
   * Create an instance. 
   *
   * viewMenu is used by view widgets to access temporary options.
   **/
  QualendarViewStack(QWidget *parent, const char* name, 
		     QArray<QualendarDB*>, QPopupMenu *);
  bool mayClose();
  QString currentViewName() const;
  QValueList<QPixmap> icons() const;
  QStringList trViewNames() const;
  /**
   * Called when "new event" is selected from the menu, etc.
   **/
  void selectedDateTime(QDateTime &, QDateTime &);
  void gotoView(const QString &);
  void gotoView(int i);
  void nextView();
  void prevView();

  /**
   * Set the date. 
   * When you don't want it to redraw(e.g. just after creation), 
   * set draw = false.
   **/
  void setDate(QDate, bool draw = true);
  /**
   * Set the filter.
   * When you don't want it to redraw(e.g. just after creation), 
   * set draw = false.
   **/
  //  void setFilter(QMap<int, bool>, bool draw = true);
  void setFilter(int i, bool draw = true);
  QDate currentDate();
  void loadConfig();
  void redraw();
 protected:
  void keyPressEvent(QKeyEvent*);

 signals:
  void viewItem(QualendarItem);
  void editItem(QualendarItem);
  void removeItem(QualendarItem);
  void createEvent(QDateTime, QDateTime, QString);
  /**
   * Notify (mainly to the main window) that the view changed. 
   **/
  void viewChanged(int);

 private:
  QArray<QualendarView*> views_;
  uint iconCount;
  QBitArray dirty_;
  int currentView_;
  QPopupMenu *menu_;
};

/**
 * The abstract base class for views.
 *
 * To create a new view, you have to implement virtual functions.
 **/
class QualendarView : public QVBox {
   Q_OBJECT
 public:
   /**
    * Create an instance.
    *
    * viewMenu is available to get options from the user.
    **/
  QualendarView(QWidget *parent, const char *name, 
		QArray<QualendarDB *>, 
		QPopupMenu *viewMenu);
   /**
    * The name of the view, such as "Month" or "WeekLandscape".
    **/
  virtual QString viewName() const = 0;
  virtual QString trViewName() const = 0;
  virtual QPixmap icon() const = 0;
  virtual QDate currentDate() const = 0; 
  virtual void selectedDateTime(QDateTime &, QDateTime &) const = 0;
  virtual void setupMenu() = 0;

 protected:
  QualendarEventDB *db;
  QualendarTaskDB *taskDB;
  QPopupMenu *menu;

 public slots:
  virtual void setFilter(QMap<int, bool>, bool draw = true) = 0;
  virtual void setDate(const QDate &, bool draw = true) = 0;
  virtual void loadConfig() = 0;
  virtual void redraw() = 0;

  //  void slotWeekChange(bool);
  //  void slotEventSelected();

 signals:
  /**
   * Request for showing an event.
   **/
  void viewItem(QualendarItem);
  /**
   * Request for editing an event.
   **/
  void editItem(QualendarItem);
  /**
   * Request for removal of an event.
   **/
  void removeItem(QualendarItem);
  /**
   * Request for creation of an event 
   * with the given start, end and description.
   **/
  void createEvent(QDateTime, QDateTime, QString);

  void dateChanged(const QDate &);

 protected:
  /* helper functions */
  QColor getColor(const Event &e);
  QString timeString(const EffectiveEvent &);
  QString dayName(int);
  QString dateString(const QualendarTask &, bool showStart, bool showDue);
  QString getIcon(const QString &, int, QPixmap &);
  QColor getDayColor(const QDate &);
  QColor getTodoColor(const QualendarTask &, QDate);
  QPixmap getTodoPixmap(const QualendarTask &, QDate, int);

  QValueList<QualendarTask> filterTasks(const QValueList<QualendarTask> &, const QMap<int, bool> &);

  /**
   * get tasks according to the options in DateBookGlobal.
   **/
  QValueList<QualendarTask> getTasks(QDate sd, QDate ed, const QMap<int, bool> &);
 private:
  QPixmap todoPixmap0, todoPixmap1, todoPixmap2;
};


#endif // QUALENDARVIEW_H
