#include "config.h"
#ifdef DBISBOX 

/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include <sl/slzdb.h>
#include <sl/slzdtm.h>
#include <qfile.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qtextstream.h>
#include <qtl.h>
#include <qmessagebox.h>
#include <qmap.h>
#include <qtimer.h>
#include <qapplication.h>

#include <qpe/global.h>
#include <qpe/stringutil.h>
#include <qpe/timeconversion.h>
#include <qpe/event.h>

#include "qualendareventdbbox.h"

QualendarEventDBBox::QualendarEventDBBox() {
  load();
}

void QualendarEventDBBox::reload() {
  
  for (QValueList<Event>::Iterator it = eventList.begin(); 
       it != eventList.end(); 
       ++it) 
    if ((*it).hasAlarm())
      delEventAlarm(*it);
  for (QValueList<Event>::Iterator it = repeatEvents.begin();
       it != repeatEvents.end(); 
       ++it) 
    if ((*it).hasAlarm()) 
      delEventAlarm(*it);

  eventList.clear();
  eventCardIds.clear();
  repeatEvents.clear(); 
  repeatCardIds.clear();
  load();

  emitSignal();
}

bool QualendarEventDBBox::save() {
  // Nothing to do, since we have done it for each modification.
  return true;
}

QValueList<EffectiveEvent> 
QualendarEventDBBox::getEffectiveEvents(const QDate &from, const QDate &to) {
  QValueList<EffectiveEvent> tmpList;
  QValueListIterator<Event> it;

  EffectiveEvent effEv;
  QDateTime dtTmp, dtEnd;

  for (it = eventList.begin(); it != eventList.end(); ++it) {
    dtTmp = (*it).start(TRUE);
    dtEnd = (*it).end(TRUE);

    if (dtTmp.date() >= from && dtTmp.date() <= to)  {
      Event tmpEv = *it;
      effEv.setEvent(tmpEv);
      effEv.setDate(dtTmp.date());
      effEv.setStart(dtTmp.time());
      if (dtTmp.date() != dtEnd.date())
	effEv.setEnd(QTime(23, 59, 59));
      else
	effEv.setEnd(dtEnd.time());
      tmpList.append(effEv);
    }
    // we must also check for end date information...
    if (dtEnd.date() != dtTmp.date() && dtEnd.date() >= from) {
      QDateTime dt = dtTmp.addDays(1);
      dt.setTime(QTime(0, 0, 0));
      QDateTime dtStop;
      if (dtEnd > to) {
	dtStop = to;
      } else
	dtStop = dtEnd;
      while (dt <= dtStop) {
	Event tmpEv = *it;
	effEv.setEvent(tmpEv);
	effEv.setDate(dt.date());
	if (dt >= from) {
	  effEv.setStart(QTime(0, 0, 0));
	  if (dt.date() == dtEnd.date())
	    effEv.setEnd(dtEnd.time());
	  else
	    effEv.setEnd(QTime(23, 59, 59));
	  tmpList.append(effEv);
	}
	dt = dt.addDays(1);
      }
    }
  }
  // check for repeating events...
  QDateTime repeat;
  for (it = repeatEvents.begin(); it != repeatEvents.end(); ++it) {
    /* create a false end date, to short circuit on hard
       MonthlyDay recurrences */
    Event dummy_event = *it;
    int duration = (*it).start().date().daysTo((*it).end().date());
    QDate itDate = from.addDays(-duration);

    Event::RepeatPattern r = dummy_event.repeatPattern();
    if (!r.hasEndDate || r.endDate() > to) {
      r.setEndDate(to);
      r.hasEndDate = TRUE;
    }
    if (dummy_event.hasRepeat()) // For safety.
      dummy_event.setRepeat(TRUE, r);

    while (nextOccurance(dummy_event, itDate, repeat)) {
      if(repeat.date() > to)
	break;
      effEv.setDate(repeat.date());
      if ((*it).type() == Event::AllDay) {
	effEv.setStart(QTime(0, 0, 0));
	effEv.setEnd(QTime(23, 59, 59));
      } else {
	/* we only occur by days, not hours/minutes/seconds.  Hence
	   the actual end and start times will be the same for
	   every repeated event.  For multi day events this is
	   fixed up later if on wronge day span */
	effEv.setStart((*it).start().time());
	effEv.setEnd((*it).end().time());
      }
      if (duration != 0) {
	// multi-day repeating events
	QDate sub_it = QMAX(repeat.date(), from);
	QDate startDate = repeat.date();
	QDate endDate = startDate.addDays(duration);

	while (sub_it <= endDate && sub_it <= to) {
	  EffectiveEvent tmpEffEv = effEv;
	  Event tmpEv = *it;
	  tmpEffEv.setEvent(tmpEv);

	  if (sub_it != startDate)
	    tmpEffEv.setStart(QTime(0, 0, 0));
	  if (sub_it != endDate)
	    tmpEffEv.setEnd(QTime(23, 59, 59));
	  tmpEffEv.setDate(sub_it);
	  tmpEffEv.setEffectiveDates(startDate, endDate);
	  tmpList.append(tmpEffEv);
	  sub_it = sub_it.addDays(1);
	}
	itDate = endDate;
      } else {
	Event tmpEv = *it;
	effEv.setEvent(tmpEv);
	tmpList.append(effEv);
	itDate = repeat.date().addDays(1);
      }
    }
  }

  qHeapSort(tmpList);
  return tmpList;
}

QValueList<EffectiveEvent> 
QualendarEventDBBox::getEffectiveEvents(const QDateTime &dt) {
  QValueList<EffectiveEvent> day = getEffectiveEvents(dt.date(), dt.date());
  QValueListConstIterator<EffectiveEvent> it;
  QValueList<EffectiveEvent> tmpList;
  QDateTime dtTmp;

  for (it = day.begin(); it != day.end(); ++it ) {
    dtTmp = QDateTime((*it).date(), (*it).start());
    // at the moment we don't have second granularity, be nice about that..
    if (QABS(dt.secsTo(dtTmp)) < 60) 
      tmpList.append(*it);
  }
  return tmpList;
}

QValueList<Event> QualendarEventDBBox::getRawRepeats() const {
  return repeatEvents;
}

// Must be rewritten.
QValueList<Event> 
QualendarEventDBBox::getNonRepeatingEvents(const QDate &from,
					   const QDate &to) const {
  QValueListConstIterator<Event> it;
  QDateTime dtTmp, dtEnd;
  QValueList<Event> tmpList;
  for (it = eventList.begin(); it != eventList.end(); ++it ) {
    if ((*it).start().date() <= to && (*it).end().date() >= from)
      tmpList.append(*it);
    /*
    dtTmp = (*it).start(TRUE);
    dtEnd = (*it).end(TRUE);

    if (dtTmp.date() >= from && dtTmp.date() <= to) {
      Event e = *it;
      if (dtTmp.date() != dtEnd.date())
	e.setEnd(QDateTime(dtTmp.date(), QTime(23, 59, 59)));
      tmpList.append(e);
    }
    // we must also check for end date information...
    if (dtEnd.date() != dtTmp.date() && dtEnd.date() >= from) {
      QDateTime dt = dtTmp.addDays(1);
      dt.setTime(QTime(0, 0, 0));
      QDateTime dtStop;
      if (dtEnd > to) {
	dtStop = to;
      } else
	dtStop = dtEnd;
      while (dt <= dtStop) {
	Event ev = *it;
	if (dt >= from) {
	  ev.setStart(QDateTime(dt.date(), QTime(0, 0, 0)));
	  if (dt.date() == dtEnd.date())
	    ev.setEnd(QDateTime(dt.date(), dtEnd.time()));
	  else
	    ev.setEnd(QDateTime(dt.date(), QTime(23, 59, 59)));
	  tmpList.append(ev);
	}
	dt = dt.addDays(1);
      }
    }
    */
  }
  return tmpList;
}

void QualendarEventDBBox::add(const Event &ev) {
  SlZDataBase *zdb = openForWrite();
  if (!zdb)
    return;

  CardId cardId = 0;
  bool result = zdb->addDatebookEntry(zdb, &cardId, ev);
  //  qDebug("addEvent %d", (int) cardId);

  if (result) {
    Event e = toEvent(zdb, cardId); 
    e.assignUid(); 
    if (zdb->checkGarbageAlert(SlZDataBase::datebookFileName()))
      emit exception("Information", 
		     "Garbage collection for Calendar performed.");
    addEventRaw(e, cardId);

    emitSignal();
  } else {
    emit exception("Error", 
		   tr("THIS MIGHT BE A BUG.\n"
		      "Couldn't add an event."));
  }

  zdb->close();
  delete zdb;
}

void QualendarEventDBBox::edit(const Event &old, const Event &editedEv) {
  SlZDataBase *zdb = openForWrite();
  if (!zdb)
    return;

  CardId cardId;
  int oldIndex;

  if (!findCard(old, oldIndex, cardId, "edit")) {
    zdb->close();
    delete zdb;
    return;
  }

  //  qDebug("editEvent %d", (int) cardId);
  bool result = zdb->editDatebookEntry(zdb, &cardId, editedEv); 
  if (result) {
    if (old.hasAlarm())
      delEventAlarm(old);
    if (old.hasRepeat()) {
      repeatEvents.remove(repeatEvents.at(oldIndex));
      repeatCardIds.remove(repeatCardIds.at(oldIndex));
    } else {
      eventList.remove(eventList.at(oldIndex));
      eventCardIds.remove(eventCardIds.at(oldIndex));
    }
    Event e = toEvent(zdb, cardId); 
    e.setUid(old.uid());
    addEventRaw(e, cardId);
    emitSignal();
  } else {
    emit exception("Error", 
		   tr("THIS MIGHT BE A BUG.\n"
		      "Error in edit(Event).\n"
		      "cardId=%1")
		   .arg(cardId));
  }

  zdb->close();
  delete zdb;
}

void QualendarEventDBBox::remove(const Event &ev) {
  SlZDataBase *zdb = openForWrite();
  if (!zdb)
    return;

  CardId cardId;
  int i;

  if (!findCard(ev, i, cardId, "remove")) {
    zdb->close();
    delete zdb;
    return;
  }

  // qDebug("removeEvent %d", cardId);
  bool result = zdb->deleteCard(&cardId);

  if (result) {
    if (ev.hasAlarm())
      delEventAlarm(ev);
    if (ev.hasRepeat()) {
      repeatEvents.remove(repeatEvents.at(i));
      repeatCardIds.remove(repeatCardIds.at(i));
    } else {
      eventList.remove(eventList.at(i));
      eventCardIds.remove(eventCardIds.at(i));
    }

    emitSignal();
  } else {
    emit exception("Error", 
		   tr("THIS MIGHT BE A BUG.\n"
		      "Error in remove(Event).\n"
		      "cardId=%1")
		   .arg(cardId));
  }

  zdb->close();
  delete zdb;
}

/* Calculating the next event of a recuring event is actually
   computationally inexpensive, esp. compared to checking each day
   individually.  There are bad worse cases for say the 29th of
   february or the 31st of some other months.  However
   these are still bounded */
bool QualendarEventDBBox::nextOccurance(const Event &e, const QDate &from, QDateTime &next)
{
    // easy checks, first are we too far in the future or too far in the past?
    QDate tmpDate;
    int freq = e.repeatPattern().frequency;
    int diff, diff2, a;
    int iday, imonth, iyear;
    int dayOfWeek = 0;
    int firstOfWeek = 0;
    int weekOfMonth;

    if (e.repeatPattern().type == Event::NoRepeat)
      return false;
    if (e.repeatPattern().hasEndDate && e.repeatPattern().endDate() < from)
	return FALSE;

    if (e.start() >= from) {
	next = e.start();
	return TRUE;
    }
    switch ( e.repeatPattern().type ) {
	case Event::Weekly:
	    /* weekly is just daily by 7 */
	    /* first convert the repeatPattern.Days() mask to the next
	       day of week valid after from */
            dayOfWeek = from.dayOfWeek();
	    dayOfWeek--; /* we want 0-6, doco for above specs 1-7 */

	    /* this is done in case freq > 1 and from in week not
	       for this round */
	    // firstOfWeek = 0; this is already done at decl.
	    while(!((1 << firstOfWeek) & e.repeatPattern().days))
		firstOfWeek++;

	    /* there is at least one 'day', or there would be no event */
	    while(!((1 << (dayOfWeek % 7)) & e.repeatPattern().days))
		dayOfWeek++;

	    dayOfWeek = dayOfWeek % 7; /* the actual day of week */
	    dayOfWeek -= e.start().date().dayOfWeek() -1;

	    firstOfWeek = firstOfWeek % 7; /* the actual first of week */
	    firstOfWeek -= e.start().date().dayOfWeek() -1;

	    // dayOfWeek may be negitive now
	    // day of week is number of days to add to start day

	    freq *= 7;
	    // FALL-THROUGH !!!!!
	case Event::Daily:
	    // the add is for the possible fall through from weekly */
	    if(e.start().date().addDays(dayOfWeek) > from) {
		/* first week exception */
		next = QDateTime(e.start().date().addDays(dayOfWeek),
				 e.start().time());
		if ((next.date() > e.repeatPattern().endDate())
		    && e.repeatPattern().hasEndDate)
		    return FALSE;
		return TRUE;
	    }
	    /* if from is middle of a non-week */

	    diff = e.start().date().addDays(dayOfWeek).daysTo(from) % freq;
	    diff2 = e.start().date().addDays(firstOfWeek).daysTo(from) % freq;

	    if(diff != 0)
		diff = freq - diff;
	    if(diff2 != 0)
		diff2 = freq - diff2;
	    diff = QMIN(diff, diff2);

	    next = QDateTime(from.addDays(diff), e.start().time());
	    if ( (next.date() > e.repeatPattern().endDate())
		 && e.repeatPattern().hasEndDate )
		return FALSE;
	    return TRUE;
	case Event::MonthlyDay:
	    iday = from.day();
	    iyear = from.year();
	    imonth = from.month();
	    /* find equivelent day of month for this month */
	    dayOfWeek = e.start().date().dayOfWeek();
	    weekOfMonth = (e.start().date().day() - 1) / 7;

	    /* work out when the next valid month is */
	    a = from.year() - e.start().date().year();
	    a *= 12;
	    a = a + (imonth - e.start().date().month());
	    /* a is e.start()monthsFrom(from); */
	    if(a % freq) {
		a = freq - (a % freq);
		imonth = from.month() + a;
		if (imonth > 12) {
		    imonth--;
		    iyear += imonth / 12;
		    imonth = imonth % 12;
		    imonth++;
		}
	    }
	    /* imonth is now the first month after or on
	       from that matches the frequency given */

	    /* find for this month */
	    tmpDate = QDate( iyear, imonth, 1 );

	    iday = 1;
	    iday += (7 + dayOfWeek - tmpDate.dayOfWeek()) % 7;
	    iday += 7 * weekOfMonth;
	    while (iday > tmpDate.daysInMonth()) {
		imonth += freq;
		if (imonth > 12) {
		    imonth--;
		    iyear += imonth / 12;
		    imonth = imonth % 12;
		    imonth++;
		}
		tmpDate = QDate( iyear, imonth, 1 );
		/* these loops could go for a while, check end case now */
		if ((tmpDate > e.repeatPattern().endDate()) && e.repeatPattern().hasEndDate)
		    return FALSE;
		iday = 1;
		iday += (7 + dayOfWeek - tmpDate.dayOfWeek()) % 7;
		iday += 7 * weekOfMonth;
	    }
	    tmpDate = QDate(iyear, imonth, iday);

	    if (tmpDate >= from) {
		next = QDateTime(tmpDate, e.start().time());
		if ((next.date() > e.repeatPattern().endDate()) && e.repeatPattern().hasEndDate)
		    return FALSE;
		return TRUE;
	    }

	    /* need to find the next iteration */
	    do {
		imonth += freq;
		if (imonth > 12) {
		    imonth--;
		    iyear += imonth / 12;
		    imonth = imonth % 12;
		    imonth++;
		}
		tmpDate = QDate( iyear, imonth, 1 );
		/* these loops could go for a while, check end case now */
		if ((tmpDate > e.repeatPattern().endDate()) && e.repeatPattern().hasEndDate)
		    return FALSE;
		iday = 1;
		iday += (7 + dayOfWeek - tmpDate.dayOfWeek()) % 7;
		iday += 7 * weekOfMonth;
	    } while (iday > tmpDate.daysInMonth());
	    tmpDate = QDate(iyear, imonth, iday);

	    next = QDateTime(tmpDate, e.start().time());
	    if ((next.date() > e.repeatPattern().endDate()) && e.repeatPattern().hasEndDate)
		return FALSE;
	    return TRUE;
	case Event::MonthlyDate:
	    iday = e.start().date().day();
	    iyear = from.year();
	    imonth = from.month();

	    a = from.year() - e.start().date().year();
	    a *= 12;
	    a = a + (imonth - e.start().date().month());
	    /* a is e.start()monthsFrom(from); */
	    if(a % freq) {
		a = freq - (a % freq);
		imonth = from.month() + a;
		if (imonth > 12) {
		    imonth--;
		    iyear += imonth / 12;
		    imonth = imonth % 12;
		    imonth++;
		}
	    }
	    /* imonth is now the first month after or on
	       from that matches the frequencey given */

	    /* this could go for a while, worse case, 4*12 iterations, probably */
	    while(!QDate::isValid(iyear, imonth, iday) ) {
		imonth += freq;
		if (imonth > 12) {
		    imonth--;
		    iyear += imonth / 12;
		    imonth = imonth % 12;
		    imonth++;
		}
		/* these loops could go for a while, check end case now */
		if ((QDate(iyear, imonth, 1) > e.repeatPattern().endDate()) && e.repeatPattern().hasEndDate)
		    return FALSE;
	    }

	    if(QDate(iyear, imonth, iday) >= from) {
		/* done */
		next = QDateTime(QDate(iyear, imonth, iday),
			e.start().time());
		if ((next.date() > e.repeatPattern().endDate()) && e.repeatPattern().hasEndDate)
		    return FALSE;
		return TRUE;
	    }

	    /* ok, need to cycle */
	    imonth += freq;
	    imonth--;
	    iyear += imonth / 12;
	    imonth = imonth % 12;
	    imonth++;

	    while(!QDate::isValid(iyear, imonth, iday) ) {
		imonth += freq;
		imonth--;
		iyear += imonth / 12;
		imonth = imonth % 12;
		imonth++;
		if ((QDate(iyear, imonth, 1) > e.repeatPattern().endDate()) && e.repeatPattern().hasEndDate)
		    return FALSE;
	    }

	    next = QDateTime(QDate(iyear, imonth, iday), e.start().time());
	    if ((next.date() > e.repeatPattern().endDate()) && e.repeatPattern().hasEndDate)
		return FALSE;
	    return TRUE;
	case Event::Yearly:
	    iday = e.start().date().day();
	    imonth = e.start().date().month();
	    iyear = from.year(); // after all, we want to start in this year

	    diff = 1;
	    if(imonth == 2 && iday > 28) {
		/* leap year, and it counts, calculate actual frequency */
		if(freq % 4)
		    if (freq % 2)
			freq = freq * 4;
		    else
			freq = freq * 2;
		/* else divides by 4 already, leave freq alone */
		diff = 4;
	    }

	    a = from.year() - e.start().date().year();
	    if(a % freq) {
		a = freq - (a % freq);
		iyear = iyear + a;
	    }

	    /* under the assumption we won't hit one of the special not-leap years twice */
	    if(!QDate::isValid(iyear, imonth, iday)) {
		/* must have been skipping by leap years and hit one that wasn't, (e.g. 2100) */
		iyear += freq;
	    }

	    if(QDate(iyear, imonth, iday) >= from) {
		next = QDateTime(QDate(iyear, imonth, iday),
			e.start().time());
		if ((next.date() > e.repeatPattern().endDate()) && e.repeatPattern().hasEndDate)
		    return FALSE;
		return TRUE;
	    }
	    /* iyear == from.year(), need to advance again */
	    iyear += freq;
	    /* under the assumption we won't hit one of the special not-leap years twice */
	    if(!QDate::isValid(iyear, imonth, iday)) {
		/* must have been skipping by leap years and hit one that wasn't, (e.g. 2100) */
		iyear += freq;
	    }

	    next = QDateTime(QDate(iyear, imonth, iday), e.start().time());
	    if ((next.date() > e.repeatPattern().endDate()) && e.repeatPattern().hasEndDate)
		return FALSE;
	    return TRUE;
	default:
	    return FALSE;
    }
}

void QualendarEventDBBox::load() {
  QTime t;
  t.start();

  qDebug("QualendarEventDBBox: opening SlZDataBase... %d", t.elapsed());

  SlZDataBase *zdb;
  zdb = new SlZDataBase(SlZDataBase::datebookFileName(),
			SlZDataBase::datebookItems(), 
			NULL, true);
  if (!zdb || !zdb->isOpened()) {
    QMessageBox::critical(0, "Can't open Calendar Database", 
			  tr("Use preinstalled Calendar\n"
			     "to create the data file"));
    qApp->quit();
  }

  qDebug("QualendarEventDBBox: reading data... %d", t.elapsed());
  int i = 0;
  for(CardId cardId = zdb->firstCard(); 
      cardId != 0;
      cardId = zdb->nextCard(cardId)) {
    Event e = toEvent(zdb, cardId);
    e.assignUid(); 
    if (!addEventRaw(e, cardId))
      break;
    i++;
  }

  qDebug("QualendarEventDBBox: there are %d events", i);
  qDebug("QualendarEventDBBox: done %d", t.elapsed() );

  zdb->close();
  delete zdb;

  dataDirty = false;
}

Event QualendarEventDBBox::toEvent(SlZDataBase *zdb, CardId cardId) {
  QArray<int> categories;
  QString description, location, note;
  QString timezone;
  QDateTime start, end;
  uchar allDay;
  uchar hasAlarm;
  int alarmTime;
  uchar soundType;
  signed char type;
  short frequency;
  ushort position;
  uchar days;
  ulong rend;
  QDate rEDT;
  QDate aLSD, aLED;
  uchar mday;
  // ... for the alarm settings ...
  alarmTime = -1; Event::SoundTypeChoice alarmSound = Event::Silent;
  // ... and for the recurrence
  Event::RepeatPattern rp;
  Event e;

  categories = zdb->readCategories(cardId); 
  description = zdb->readField(ZdbDate::Description, cardId); 
  location = zdb->readField(ZdbDate::Location, cardId); 
  note = zdb->readField(ZdbDate::Note, cardId); 
  timezone = zdb->readField(ZdbDate::TimeZone, cardId); 

  allDay = zdb->readUcharField(ZdbDate::Type, cardId);
  hasAlarm = (zdb->readUcharField(ZdbDate::Alarm, cardId) 
	      == ZdbDate::AlarmOn) ? true:false;
  if (hasAlarm)
    alarmTime = zdb->readUshortField(ZdbDate::AlarmTime, cardId);
  soundType = zdb->readUcharField(ZdbDate::SoundType, cardId);
  if (soundType) 
    alarmSound = Event::Loud;
  type = zdb->readUcharField(ZdbDate::RepeatType, cardId); // type is signed
  if (type >= 0) {
    frequency = zdb->readUshortField(ZdbDate::RepeatFrequency, cardId);
    position = zdb->readUshortField(ZdbDate::RepeatPosition, cardId);
    days = zdb->readUcharField(ZdbDate::RepeatDays, cardId);
    rend = zdb->readUcharField(ZdbDate::RepeatHasEndDate, cardId);
    rEDT = zdb->readDateField(ZdbDate::RepeatEndDate, cardId);
  } 

  if (allDay) {
    aLSD = zdb->readDateField(ZdbDate::AllDayStart, cardId);
    aLED = zdb->readDateField(ZdbDate::AllDayEnd, cardId);
  } else {
    start = zdb->readTimeField(ZdbDate::StartDateTime, cardId);
    end = zdb->readTimeField(ZdbDate::EndDateTime, cardId);
  }

  mday = zdb->readUcharField(ZdbDate::MultiDate, cardId);

  e.setDescription(description);
  e.setLocation(location);
  int i = QDateTime(QDate(1970, 1, 2), QTime(0, 0, 0))
    .secsTo(TimeConversion::fromUTC(86400));
  e.setStart(allDay ? QDateTime(aLSD, QTime(0, 0, 0)) : 
	     start.addSecs(i));
  e.setEnd(allDay ? QDateTime(aLED, QTime(23, 59, 0)) : 
	   end.addSecs(i));
  e.setCategories(categories);
  if (allDay) 
    e.setType(Event::AllDay);
  else
    e.setType(Event::Normal);
  rp.type = (Event::RepeatType) type;
  rp.days = days;
  rp.position = position;
  rp.frequency = frequency;
  rp.hasEndDate = rend;
  if (rend) {
    QDateTime dt;
    dt.setTime_t(0); 
    rp.endDateUTC = dt.secsTo(rEDT);
  }
  //  e.setStart( (time_t) value.toLong() );
  e.setNotes(note);

  // "post processing" (dates, times, alarm, recurrence)
  // start date/time
  e.setRepeat( rp.type != Event::NoRepeat, rp );

  if ( alarmTime != -1 )
    e.setAlarm( TRUE, alarmTime, alarmSound );

  return e;
}

bool QualendarEventDBBox::addEventRaw(const Event &ev, CardId cardId) {
  if (ev.uid() == 0 || cardId == 0 || 
      (!ev.hasRepeat() && 
       (eventCardIds.contains(cardId))) || 
      (ev.hasRepeat() &&
       (repeatCardIds.contains(cardId))) ||
      eventList.count() != eventCardIds.count() ||
      repeatEvents.count() != repeatCardIds.count()) {
    // Let's not check ev is already contained.

    emit exception("Error", 
		   tr("THIS MIGHT BE A BUG.\n"
		      "Error in addEventRaw.\n"
		      "uid=%1, cardId=%2,\n"
		      "eventList.count()=%3, eventCardIds.count()=%4,\n"
		      "repeatEvents.count()=%5, repeatCardIds.count()=%6")
		   .arg(ev.uid()).arg(cardId)
		   .arg(eventList.count()).arg(eventCardIds.count())
		   .arg(repeatEvents.count()).arg(repeatCardIds.count()));
    return false;
  }

  if (ev.hasAlarm())
    addEventAlarm(ev);
  if (ev.hasRepeat()) {
    repeatEvents.append(ev);
    repeatCardIds.append(cardId);
  } else {
    eventList.append(ev);
    eventCardIds.append(cardId);
  }
  return true;
}

bool QualendarEventDBBox::findCard(const Event &ev, 
				   int &i, CardId &cardId, 
				   QString op) {
  if (!&ev) {
    emit exception("Error", 
		   tr("THIS MIGHT BE A BUG.\n"
		      "Null pointer in findCard, %1(Event).")
		   .arg(op));
    return false;
  }

  if (ev.hasRepeat()) {
    i = repeatEvents.findIndex(ev);
    if (ev.uid() == 0 || i < 0 || 
	repeatEvents.count() != repeatCardIds.count()) {
      emit exception("Error", 
		     tr("THIS MIGHT BE A BUG.\n"
			"Error in findCard, %1(Event).\n"
			"uid=%2, repeatEvents.count()=%3, "
			"repeatCardIds.count()=%4")
		     .arg(op)
		     .arg(ev.uid())
		     .arg(repeatEvents.count()).arg(repeatCardIds.count()));
      return false;
    } else {
      cardId = repeatCardIds[i];
      return true;
    } 
  } else {
    i = eventList.findIndex(ev);
    if (ev.uid() == 0 || i < 0 || eventList.count() != eventCardIds.count()) {
      emit exception("Error", 
		     tr("THIS MIGHT BE A BUG.\n"
			"Error in findCard, %1(Event).\n"
			"uid=%2, eventList.count()=%3, "
			"eventCardIds.count()=%4")
		     .arg(op)
		     .arg(ev.uid())
		     .arg(eventList.count()).arg(eventCardIds.count()));
      return false;
    } else {
      cardId = eventCardIds[i];
      return true;
    } 
  }
}

SlZDataBase *QualendarEventDBBox::openForWrite() {
  SlZDataBase *zdb;

  zdb = new SlZDataBase(SlZDataBase::datebookFileName(),
			SlZDataBase::datebookItems()); 
  if (zdb) {
    if (zdb->isOpened()) 
      return zdb;
    else
      delete zdb;
  }

  emit exception("Error", 
		 tr("Couldn't open DateBook database for write.\n"
		    "Stop(and disable fastloading)\n"
		    "of other applications using SLDATE.BOX"));
  return 0;
}


#endif // DBISBOX
