#include "qualendarday.h"
#include "../qualendaritem.h"
#include "../qualendarglobal.h"
#include "qualendardayheader.h"
#include "../sysdep/qualendardb.h"
#include <qpe/calendar.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qwidgetstack.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qpainter.h>
#include <qlistbox.h>
#include <qscrollbar.h>
#include <qlabel.h>
#include <qapplication.h>
#include <qpe/resource.h>
#include <qpe/config.h>

#include "../debug.h"

MyScrollView::MyScrollView(QWidget *parent, const char *name) 
  : QHBox(parent, name) {
  viewport_ = new QualendarMainArea(this);
  bar_ = new QScrollBar(Qt::Vertical, this);
  connect(bar_, SIGNAL(valueChanged(int)), 
	  this, SLOT(barMoved()));
  connect(bar_, SIGNAL(sliderReleased()), 
	  this, SLOT(barMoved()));
  lastValue = 0;
}
QualendarMainArea *MyScrollView::viewport() {
  return viewport_;
}
void MyScrollView::setContentsHeight(int h) {
  bar_->setRange(0, h - height());
  bar_->setPageStep(height());
}
int MyScrollView::currentY() {
  return bar_->value();
}
void MyScrollView::setCurrentY(int y) {
  bar_->setValue(y);
}
void MyScrollView::setLineStep(int i) {
  bar_->setSteps(i, bar_->pageStep());
}
void MyScrollView::barMoved() {
  int v = lastValue;
  lastValue = bar_->value();
  if (bar_->draggingSlider() && 
      bar_->value()/bar_->lineStep() == v/bar_->lineStep())
    return;
  viewport_->update();
}

QualendarDay::QualendarDay(QWidget *parent, const char *name, 
			   QArray<QualendarDB *> dataBases, 
			   QPopupMenu *menu) 
  : QualendarView(parent, name, dataBases, menu), menu_(menu) {

  //  loadConfig();

  header = new QualendarDayHeader(this);

  sv = new MyScrollView(this);
  sv->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,
				QSizePolicy::Expanding));


  mainArea = sv->viewport();
  selectionMode = false;

  connect(header, SIGNAL(dateChanged(const QDate &)),
          this, SLOT(setDate(const QDate &)));
  connect(mainArea, SIGNAL(paintMe(QRect)), 
	  this, SLOT(paintMainArea()));
  connect(mainArea, SIGNAL(mouseEvent(QMouseEvent *)),
          this, SLOT(slotMouseEvent(QMouseEvent *)));

  menu->clear();
  actionShowBackgroundEvent = new QAction("", tr("Show background event"), 
					  0, this, 0, true);
  actionShowBackgroundEvent->setOn(true);
  connect(actionShowBackgroundEvent, SIGNAL(activated()), 
	  this, SLOT(redraw()));
}

QString QualendarDay::viewName() const {
  return "Day";
}
QString QualendarDay::trViewName() const {
  return tr("Day");
}
QPixmap QualendarDay::icon() const {
  return Resource::loadPixmap("qualendar/day.png");
}
QDate QualendarDay::currentDate() const {
  return date_;
}
void QualendarDay::selectedDateTime(QDateTime &start, QDateTime &end) const {
  start = QDateTime(date_, QTime(10, 0, 0));
  end = QDateTime(date_, QTime(12, 0, 0));
}
void QualendarDay::setupMenu() {
  actionShowBackgroundEvent->addTo(menu_);
}

#define N_EXTRA (numHol + numTodo + numAllDay)
#define N_CELLS (numHol + numTodo + numAllDay + 24)

void QualendarDay::resizeEvent(QResizeEvent *) {
  DRAW_DEBUG(qDebug("resizeEvent %d %d", width(), height()));
  QFontMetrics fm = QPainter(this).fontMetrics();
  QWidget *d = QApplication::desktop();
  if (width() < d->width()/2) {
    //    drawReady = false;
  } else {
    //    drawReady = true;
    //    mustRelayout = true;
    cellHeight = (rowHeight>0) ? rowHeight : fm.height()+2;
    sv->setContentsHeight(cellHeight * N_CELLS);
    sv->setLineStep(cellHeight);
  }
}

void QualendarDay::keyPressEvent(QKeyEvent *e) {
  switch (e->key()) {
  case Key_Right:
    setDate(date_.addDays(1), true);
    break;
  case Key_Left:
    setDate(date_.addDays(-1), true);
    break;
  case Key_Up:
    if (currentItem > 0) {
      currentItem--;
      if (currentItem >= itemRects0.count()) {
	QRect r = itemRects[currentItem - itemRects0.count()];
	if (r.y() < sv->currentY()) 
	  sv->setCurrentY(QMIN(r.y(), cellHeight * N_CELLS - sv->height()));
	else 
	  mainArea->update();
      } else
	mainArea->update();
    }
    break;
  case Key_Down:
    if (currentItem < itemRects0.count() + itemRects.count() - 1) {
      currentItem++;
      if (currentItem >= itemRects0.count()) {
	QRect r = itemRects[currentItem - itemRects0.count()];
	if (r.y() + r.height() >  sv->currentY() + sv->height()) 
	  sv->setCurrentY(QMIN(r.y() + r.height() + cellHeight * N_EXTRA - sv->height(), r.y()));
	else 
	  mainArea->update();
      } else
	mainArea->update();
    }
    break;

#ifndef VIEWONLY
  case Key_E:
  case 0x201a: // Fn + M
    emit editItem(selectedItem());
    break;
  case Key_Delete:
    emit removeItem(selectedItem());
    break;
#endif
  case Key_V:
  case Key_Return:
  case Key_Space:
    if ((e->key() == Key_Return && QualendarGlobal::spaceReturn) ||
	(e->key() == Key_Space && !QualendarGlobal::spaceReturn))
      e->ignore();
    else
      emit viewItem(selectedItem());
    break;
  default:
    e->ignore();
  }
}

#define STEPS 2
#define QUANTIZE(y) ((y/cellHeight)*STEPS + \
((y-(y/cellHeight)*cellHeight)/(cellHeight/STEPS)))

void QualendarDay::slotMouseEvent(QMouseEvent *e) {
  switch(e->type()) {
  case QEvent::MouseButtonPress:
    for(uint i=0; i<itemRects0.count(); i++) {
      if (itemRects0[i].contains(e->pos())) {
	openItemRect(itemRects0[i]);
	return;
      }
    }
    for(uint i=0; i<itemRects.count(); i++) {
      if (itemRects[i].contains(e->pos() + QPoint(0, sv->currentY() - N_EXTRA*cellHeight))) {
	openItemRect(itemRects[i]);
	return;
      }
    }
    {
      if (e->pos().y() < N_EXTRA * cellHeight || 
	  e->pos().y() >= mainArea->height())
	return;
      int y = e->pos().y() + sv->currentY() - N_EXTRA * cellHeight;
      selectionMode = true;
      selectionOrig = QUANTIZE(y);
      selectionStart = selectionOrig;
      selectionEnd = QUANTIZE(y) + 1;
      mainArea->update();
    }
  break;
  case QEvent::MouseMove:
    {
      if (e->pos().y() < N_EXTRA * cellHeight || 
	  e->pos().y() >= mainArea->height()) {
	selectionMode = false;
	mainArea->update();
	return;
      }
      int y = e->pos().y() + sv->currentY() - N_EXTRA * cellHeight;
      int a = QUANTIZE(y);
      if (selectionStart != QMIN(a, selectionOrig) ||
	  selectionEnd != QMAX(a, selectionOrig) + 1) {
	selectionStart = QMIN(a, selectionOrig);
	selectionEnd = QMAX(a, selectionOrig) + 1;
	mainArea->update();
      }
    }
    break;
  case QEvent::MouseButtonRelease:
    if (selectionMode) {
      selectionMode = false;
      int e = selectionEnd*3600/STEPS;
      if (e >= 86400)
	e = 86340;
      emit createEvent(QDateTime(date_, QTime(0, 0, 0).addSecs(selectionStart*3600/STEPS)), 
		       QDateTime(date_, QTime(0, 0, 0).addSecs(e)), 
		       "");
    }
    break;
  default:
    break;
  }
}

void QualendarDay::loadConfig() {
  header->setFontSize(QualendarGlobal::fontSize);

  Config config("Qualendar");
  config.setGroup("Day");
  fontSize = config.readNumEntry("FontSize", QualendarGlobal::fontSize);
  if (fontSize < 0) 
    fontSize = font().pixelSize();
  todoStart = config.readBoolEntry("TodoStart", true);
  todoDue = config.readBoolEntry("TodoDue", true);
  rowHeight = config.readNumEntry("RowHeight", -1);
  minimalEventHeight = config.readNumEntry("MinimalEventHeight", 60);
  int c = config.readNumEntry("SelectedColor", 0xc0c0ff);
  selectedColor = QColor((c & 0xff0000)>>16, (c & 0xff00) >> 8, c & 0xff);
}

void QualendarDay::setFilter(QMap<int, bool> filter, bool draw) {
  filter_ = filter;
  //  tableChanged = true;
  //  mustRelayout = true;
  if (draw) {
    DRAW_DEBUG(qDebug("Draw from setFilter"));
    redraw();
  }
}

void QualendarDay::setDate(const QDate &d, bool mustDraw) {
  date_ = d;
  header->setDate(d);
  dr.setDate(d);
  getEvents();
  relayout();
  if (mustDraw) {
    mainArea->update();
  }
}
void QualendarDay::setDate(const QDate &d) {
  setDate(d, true);
}

void QualendarDay::redraw() {
  header->setLabels();
  header->setDate(date_);
  //  loadConfig();
  getEvents();
  relayout();
  mainArea->update();
}

typedef struct {
  int i;
  int t;
} StripData;

void QualendarDay::relayout() {
  DRAW_DEBUG(qDebug("QualendarDay::relayout"));
#ifdef DEBUG_PROFILE
  QTime tm;
  tm.start();
#endif
  itemRects0.clear();
  itemRects.clear();
  currentItem = 0;

  QWidget *w = mainArea;
  if (fontSize >= 0) {
    w->setFont(QFont(w->font().family(), fontSize));
  }
  QPainter p(w);
  QFontMetrics fm = p.fontMetrics();
  // 
  headerWidth = fm.width("00:00p") + 2;
  cellWidth = w->width() - headerWidth;
  cellHeight = (rowHeight>0) ? rowHeight : fm.height()+2;

  // load pixmaps here.
  //  QImage im = Resource::loadPixmap("qualendar/todo").convertToImage();

  upPixmap = Resource::loadPixmap("qualendar/up").convertToImage()
    .smoothScale(fm.height(), fm.height());
  downPixmap = Resource::loadPixmap("qualendar/down").convertToImage()
    .smoothScale(fm.height(), fm.height());
  //  repeatPixmap = Resource::loadPixmap("qualendar/repeat").convertToImage()
  //    .smoothScale(fm.height(), fm.height());
  repeatPixmap = Resource::loadPixmap("qualendar/repeat");

  // holiday? 
  if (QualendarGlobal::holidays.contains(date_)) {
    numHol = 1;
    holDesc = QualendarGlobal::holidays[date_];
  } else {
    numHol = 0;
  }

  // construct the list of tasks.
  tl = getTasks(date_, date_, filter_);
  numTodo = tl.count();

  for(uint i = 0; i < numTodo; i++) {
    const QualendarTask &task = tl[i];
    ItemRect er;
    QColor color = getTodoColor(task, date_);
    QString s = task.description();
    QPixmap pm;

    s = getIcon(s, fontSize, pm);
    er.setFixedRect(true);
    er.setRect(0, (numHol + i)*cellHeight, w->width(), cellHeight);
    er.addPixmap(getTodoPixmap(task, date_, fontSize));
    if (!pm.isNull()) 	
      er.addPixmap(pm);
    QString t = QString(" %1 ").arg(task.priority());
    QString u = dateString(task, true, true);
    if (!u.isEmpty())
      t += "(" + u + ") ";
    er.addText(fm, t, QColor(0, 0, 0));
    er.addText(fm, s, color);

    if (!task.notes().isEmpty()) 
      er.addText(fm, " -- " + task.notes(), color);

    er.setItem(task);
    itemRects0.append(er);
  }

  // layout the day cell
  dr.setRect(headerWidth, 0, cellWidth, cellHeight*24);
  dr.setBackground(QPixmap());

  // layout events
  {
    int ix = headerWidth;

    // first, divide events into strips, and layout all-day events.
    numAllDay = 0;
    QValueList< QValueList <StripData> > strips;
    for(uint i = 0; i < dr.events().count(); i++) {
      const EffectiveEvent &eev = dr.events()[i];
      const Event &ev = eev.event();
      QPixmap pm = QualendarGlobal::getBackground(ev.description(), 
						  headerWidth, cellHeight*2);
      if (!pm.isNull()) {
	dr.setBackground(pm);
	if (!actionShowBackgroundEvent->isOn()) {
	  dr.setEventHidden(i, true);
	  continue;
	}
      } else {
	dr.setEventHidden(i, false);
      }

      if (ev.type() == Event::AllDay) {
	ItemRect er;

	if (ev.hasRepeat())
	  er.addPixmap(repeatPixmap);

	QColor color = getColor(ev);

	QString s = ev.description();
	QPixmap pm;

	s = getIcon(s, fontSize, pm);
	er.setFixedRect(true);
	er.setRect(0, (numHol + numTodo + numAllDay)*cellHeight, 
		   w->width(), cellHeight);
	if (!pm.isNull()) 	
	  er.addPixmap(pm);
	er.addText(fm, s, color);

	if (!ev.location().isEmpty())
	  er.addText(fm, QString(" (%1)").arg(ev.location()), color);

	if (!ev.notes().isEmpty())
	  er.addText(fm, " -- " + ev.notes(), color);

	er.setItem(eev);
	itemRects0.append(er);
	numAllDay++;
	continue;
      }

      uint j;

      QTime t0(0, 0, 0);
      for(j = 0; j < strips.count(); j++) {
	if (strips[j].last().t <= t0.secsTo(eev.start()) + 60) {
	  StripData d;
	  d.i = i;
	  d.t = QMAX(t0.secsTo(eev.end()), t0.secsTo(eev.start()) + 60 * minimalEventHeight);
	  strips[j].append(d);
	  break;
	}
      }
      if (j == strips.count()) {
	QValueList<StripData> l;
	StripData d;
	d.i = i;
	d.t = QMAX(t0.secsTo(eev.end()), t0.secsTo(eev.start()) + 60 * minimalEventHeight);
	l.append(d);
	strips.append(l);
      }
    }

    uint minY = 24 * cellHeight, maxY = 0;

    int row[dr.events().count()];
    for(uint l = 0; l < strips.count(); l++) 
      for(uint k = 0; k < strips[l].count(); k++) 
	row[strips[l][k].i] = l;
    for(uint i=0; i < dr.events().count(); i++) {
      const EffectiveEvent &eev = dr.events()[i];
      const Event &ev = eev.event();
      if (ev.type() == Event::AllDay || dr.eventHidden(i))
	continue;

      ItemRect er;

      QTime t0(0, 0, 0); 
      int i0 = t0.secsTo(eev.start()), 
	i1 = QMAX(t0.secsTo(eev.end()), t0.secsTo(eev.start()) + 60 * minimalEventHeight);
      er.setFixedRect(true);
      er.setRect(ix + cellWidth*row[i]/strips.count(), 
		 i0*cellHeight/3600, 
		 cellWidth/strips.count() - 2, 
		 (i1-i0)*cellHeight/3600);

      if (ev.hasRepeat())
	er.addPixmap(repeatPixmap);

      QColor color = getColor(ev);

      QString s = ev.description();
      QPixmap pm;

      s = getIcon(s, fontSize, pm);
      if (!pm.isNull()) 
	er.addPixmap(pm);
      if (!s.isEmpty())
	er.addText(fm, s, color);

      if (!ev.location().isEmpty()) {
	if (er.currentY() + fm.height()*2 >= er.height() ||
	    (!ev.notes().isEmpty() && 
	     er.currentY() + fm.height()*3 >= er.height() &&
	     er.currentX() + fm.width("nnnnnn") < er.width())) 
	  er.addText(fm, QString(" (%1)").arg(ev.location()), color);
	else
	  er.addText(fm, tr("\n  At: ") + ev.location(), color);
      }

      if (!ev.notes().isEmpty()) {
	if (er.currentY() + fm.height()*2 >= er.height())
	  er.addText(fm, " -- " + ev.notes(), color);
	else
	  er.addText(fm, tr("\n  Notes: ") + ev.notes(), color);
      }
      er.setItem(eev);
      /*
      QTime t0(0, 0, 0); 
      int i0 = t0.secsTo(eev.start()), 
	i1 = QMAX(t0.secsTo(eev.end()), t0.secsTo(eev.start()) + 60 * minimalEventHeight);
      er.setRect(ix + cellWidth*row[i]/strips.count(), 
		 i0*cellHeight/3600, 
		 cellWidth/strips.count() - 2, 
		 (i1-i0)*cellHeight/3600);
      */
      minY = QMIN(minY, i0*cellHeight/3600);
      maxY = QMAX(maxY, i0*cellHeight/3600 + cellHeight);

      itemRects.append(er);
    }

    sv->setContentsHeight(cellHeight * N_CELLS);
    if (minY < QualendarGlobal::startTime * cellHeight) 
      sv->setCurrentY(minY);
    else if (maxY < QualendarGlobal::startTime * cellHeight + 
	     sv->height() - N_EXTRA * cellHeight)
      sv->setCurrentY(QualendarGlobal::startTime * cellHeight);
    else
      sv->setCurrentY(QMIN(minY, maxY - sv->height() + N_EXTRA * cellHeight));
  }


#ifdef DEBUG_PROFILE
  qDebug("QualendarDay::relayout %d ms", tm.elapsed());
#endif
}

void QualendarDay::draw(QRect r, bool useDirty) {
#ifdef DEBUG_PROFILE
  QTime tm;
  tm.start();
#endif
  QWidget *w = mainArea;
  QPainter p(w);
  QFontMetrics fm = p.fontMetrics();

  int scrollY = sv->currentY();

  // background
  DRAW_DEBUG(qDebug("Day: Draw background"));
  QRect updatedRect;

  p.fillRect(r, QColor(255, 255, 255));
  p.setClipping(true);
  p.setClipRect(0, cellHeight * N_EXTRA, 
		w->width(), w->height() - cellHeight * N_EXTRA);
  if (selectionMode)
    p.fillRect(0, (selectionStart*cellHeight)/STEPS + cellHeight * N_EXTRA - scrollY, 
	       w->width(), ((selectionEnd - selectionStart)*cellHeight)/STEPS, 
	       QColor(0, 0, 255));

  if (!dr.background().isNull())
    p.drawPixmap(0, cellHeight * N_EXTRA, dr.background());

  p.setPen(QColor(0, 0, 0));
  p.drawLine(headerWidth-1, 0, headerWidth-1, w->height());
  for(uint i=0; i<24; i++) {
    p.drawLine(0, cellHeight * (i+N_EXTRA) - scrollY, 
	       headerWidth - 1, cellHeight * (i+N_EXTRA) - scrollY);
    bool pm = false;
    int j = i;
    if (QualendarGlobal::ampm && j >= 12) {
      pm = 1;
      if (j > 12)
	j -= 12;
    }
    p.drawText(0, cellHeight * (i+N_EXTRA) - scrollY + fm.height(), 
	       (j < 10 ? QString(" ") : QString("")) + 
	       QString::number(j) + ":00" + (pm ? QString("p") : QString("")));
  }
  p.setClipRect(r);

  // holiday
  if (numHol > 0) {
    p.setPen(QColor(255, 0, 0));
    p.drawText(0, fm.height()-1, holDesc);
    p.setPen(QColor(0, 0, 0));
    p.drawRect(0, 0, w->width(), cellHeight);
  }

  QColor c;
  c = getDayColor(dr.date());
  /*
  if (selectionMode &&
      dr.date() >= selStart && dr.date() <= selEnd)
    p.fillRect(dr, QColor(0, 0, 255));
  else {
    p.fillRect(dr, c);
    if (!dr.background().isNull())
      p.drawPixmap(dr.topLeft()+QPoint(0, cellHeaderHeight), 
		   dr.background());
  }
    */
  updatedRect |= dr;

  // event description
  p.setClipping(false);

  for(uint i = 0; i < itemRects0.count(); i++) { 
    ItemRect er = itemRects0[i];
    DRAW_DEBUG(qDebug("Draw event(all day) %d", i));
    p.fillRect(er, 
	       (i == currentItem) ? 
	       selectedColor : 
	       QColor(255, 255, 255));
    er.draw(p);
    p.setPen(QColor(0, 0, 0));
    p.drawRect(er);
    if (i == currentItem) 
      p.drawRect(er.x()+1, er.y()+1, er.width()-2, er.height()-2);
  }

  p.setClipping(true);
  p.setClipRect(0, cellHeight * N_EXTRA, 
  		width(), height() - cellHeight * N_EXTRA);
  bool up = false, down = false;
  for(uint i = 0; i < itemRects.count(); i++) { 
    ItemRect er = itemRects[i];
    DRAW_DEBUG(qDebug("Draw event %d", i));
    if (er.y() + er.height() - cellHeight/2 < scrollY)
      up = true;
    if (er.y() + cellHeight/2 > scrollY + sv->height() - cellHeight * N_EXTRA)
      down = true;
    p.fillRect(er.x(), er.y() + cellHeight * N_EXTRA - scrollY, 
	       er.width(), er.height(), 
	       (i + itemRects0.count() == currentItem) ? 
	       selectedColor : 
	       QColor(255, 255, 255));
    er.draw(p, cellHeight * N_EXTRA - scrollY);
    p.setPen(QColor(0, 0, 0));
    p.drawRect(er.x(), er.y() + cellHeight * N_EXTRA - scrollY, 
	       er.width(), er.height());
    if (i + itemRects0.count() == currentItem) 
            p.drawRect(er.x()+1, er.y() + cellHeight * N_EXTRA - scrollY + 1, 
      		 er.width()-2, er.height()-2);
  }

  p.setClipping(false);
  if (up) 
    p.drawPixmap(w->width() - upPixmap.width(), cellHeight * N_EXTRA, 
		 upPixmap);
  if (down) 
    p.drawPixmap(w->width() - downPixmap.width(), 
		 w->height() - downPixmap.height(),
		 downPixmap);

#ifdef DEBUG_PROFILE
  qDebug("QualendarDay::draw, useDirty=%d, %d ms", useDirty, tm.elapsed());
#endif
}

void QualendarDay::paintMainArea() {
  draw(QRect(0, 0, mainArea->width(), mainArea->height()), true);
}

QualendarItem QualendarDay::selectedItem() {
  uint i = currentItem;
  if (itemRects0.count() + itemRects.count() == 0)
    return QualendarItem();
  if (i < itemRects0.count()) 
    return itemRects0[i].item();
  else 
    return itemRects[i - itemRects0.count()].item();
}

void QualendarDay::slotViewOption(QAction *a) {
  QString t = a->text();
  redraw();
}

void QualendarDay::getEvents() {
  DRAW_DEBUG(qDebug("getEvents: " + viewName()));
  if (!db)
    return;

#ifdef DEBUG_PROFILE
  QTime tm;
  tm.start();
#endif
  QValueList<EffectiveEvent> eList = db->getEffectiveEvents(date_, date_);
#ifdef DEBUG_PROFILE
  qDebug("getEvents %d ms", tm.elapsed());
#endif
  dr.clear();
  for(QValueList<EffectiveEvent>::Iterator it = eList.begin();
      it != eList.end();
      ++it) {
    int i = ((*it).event().categories().count() > 0) ? 
      (*it).event().categories()[0] : QualendarGlobal::unfiledCatId;
    if (!filter_.contains(i)) 
      dr.addEvent(*it);
  }
#ifdef DEBUG_PROFILE
  qDebug("getEvents--filter, %d ms", tm.elapsed());
#endif
}

void QualendarDay::openItemRect(const ItemRect &ir) {
  emit viewItem(ir.item());
}
