#include "qualendarfind.h"
#include "../qualendaritem.h"
#include "../qualendarglobal.h"
#include "../utils/datepicker.h"
#include "../sysdep/qualendardb.h"

#include <qtl.h>
#include <qframe.h>
#include <qpopupmenu.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qlistview.h>

#include <qpe/calendar.h>
#include <qpe/resource.h>
#include <qpe/config.h>

#include "../debug.h"

class QualendarFindItem : public QListViewItem {
 public:
  QualendarFindItem(QListView *parent, QDateTime dt, QString desc, Event e)
    : QListViewItem(parent, "", desc), event(e), date(dt.date()) {
    QTime t = dt.time();
    setText(0, QString("%1/%2/%3,%4:%5")
            .arg(date.year(), 4).arg(date.month(), 2).arg(date.day(), 2)
            .arg(t.hour(), 2).arg(t.minute(), 2).replace(QRegExp(" "), "0"));
  }
  Event event;
  QDate date;
};

QualendarFind::QualendarFind(QWidget *parent, const char *name, 
			     QArray<QualendarDB *> dataBases, 
			     QPopupMenu *menu) 
  : QualendarView(parent, name, dataBases, menu), menu_(menu) {

  QFrame *f;
  QHBoxLayout *lo;

  f = new QFrame(this);
  lo = new QHBoxLayout(f);
  leKeyword = new QLineEdit(f);
  lo->addWidget(leKeyword);
  pbFind = new QPushButton(tr("Find"), f);
  lo->addWidget(pbFind);

  cbCaseSensitive = new QCheckBox(tr("Case Sensitive"), this);

  f = new QFrame(this);
  lo = new QHBoxLayout(f);
  pbStart = new QPushButton("", f);
  lo->addWidget(pbStart);
  QLabel *l = new QLabel(tr("to"), f);
  lo->addWidget(l);
  pbEnd = new QPushButton("", f);
  lo->addWidget(pbEnd);
  
  cbBackwards = new QCheckBox(tr("Search Backwards"), this);

  lvResult = new QListView(this);
  lvResult->addColumn(tr("Start"));
  lvResult->addColumn(tr("Description"));

  menu->clear();

  connect(leKeyword, SIGNAL(returnPressed()), this, SLOT(slotDoFind()));
  connect(pbFind, SIGNAL(clicked()), this, SLOT(slotDoFind()));
  connect(cbBackwards, SIGNAL(toggled(bool)), this, SLOT(slotBackwardsToggled(bool)));
  connect(pbStart, SIGNAL(clicked()), this, SLOT(slotStartDate()));
  connect(pbEnd, SIGNAL(clicked()), this, SLOT(slotEndDate()));
  connect(lvResult, SIGNAL(clicked(QListViewItem *)),
	  this, SLOT(slotEventSelected(QListViewItem *)));
  connect(lvResult, SIGNAL(returnPressed(QListViewItem *)), 
	  this, SLOT(slotEventSelected(QListViewItem *)));
}

QString QualendarFind::viewName() const {
  return "Find";
}
QString QualendarFind::trViewName() const {
  return tr("Find");
}
QPixmap QualendarFind::icon() const {
  return Resource::loadPixmap("qualendar/find.png");
}
QDate QualendarFind::currentDate() const {
  return date_;
}
void QualendarFind::selectedDateTime(QDateTime &start, QDateTime &end) const {
  start = QDateTime(date_, QTime(10, 0, 0));
  end = QDateTime(date_, QTime(12, 0, 0));
}
void QualendarFind::setupMenu() {
}

void QualendarFind::keyPressEvent(QKeyEvent *e) {
  if (e->key() == Key_Return)
    e->accept();
  else
    QualendarView::keyPressEvent(e);
}
QualendarItem QualendarFind::selectedItem() {
  QualendarFindItem *lvi = (QualendarFindItem *)(lvResult->currentItem());
  if (lvi) 
    return QualendarItem(EffectiveEvent(lvi->event, lvi->date));
  else
    return QualendarItem();
}

void QualendarFind::setFilter(QMap<int, bool> filter, bool draw) {
  filter_ = filter;
}

void QualendarFind::setDate(const QDate &d, bool mustDraw) {
  date_ = d;
  setDate2();
  setDateButtons();
}

void QualendarFind::loadConfig() {
  Config config("Qualendar");
  config.setGroup("Find");
  fontSize = config.readNumEntry("FontSize", QualendarGlobal::fontSize);
  if (fontSize < 0) 
    fontSize = font().pixelSize();
  else {
    QFont f(font().family(), fontSize);
    setFont(f);
  }
}

void QualendarFind::redraw() {
  slotDoFind();
}

void QualendarFind::slotDoFind() {
  lvResult->clear();
  QString txt = leKeyword->text();
  QDate dtStart, dtEnd;
  bool caseSensitive = cbCaseSensitive->isChecked();
  bool backwards = cbBackwards->isChecked();
  lvResult->setSorting(0, !backwards);
  //  int category;

  if (backwards) {
    dtStart = date2;
    dtEnd = date_;
  } else {
    dtStart = date_;
    dtEnd = date2;
  }
    
  QRegExp r(txt);
  r.setCaseSensitive(caseSensitive);

  QValueList<Event> repeats = db->getRawRepeats();

  QValueListConstIterator<Event> it;
  for (it = repeats.begin(); it != repeats.end(); ++it) {
    if (!filter_.contains(((*it).categories().count() > 0) ? 
			 (*it).categories()[0] : 0) &&
	(*it).match(r)) { 
      // to be sure that it occurs
      QDate start = dtStart;
      QDateTime next;
      while (db->nextOccurance(*it, start, next) && next.date() <= dtEnd) {
	QualendarFindItem *lvi = 
	  new QualendarFindItem(lvResult, 
				next, 
				(*it).description(), 
				*it);
	start = next.date().addDays(1);
      }
    }
  }

  QValueList<Event> nonRepeats;
  nonRepeats = db->getNonRepeatingEvents(dtStart, dtEnd);

  for (it = nonRepeats.begin(); it != nonRepeats.end(); ++it) {
    if (!filter_.contains(((*it).categories().count() > 0) ? 
			 (*it).categories()[0] : 0) &&
	(*it).match(r)) {
      QualendarFindItem *lvi = 
	new QualendarFindItem(lvResult, 
			      (*it).start(), 
			      (*it).description(), 
			      *it);
    }
  }
}


void QualendarFind::slotBackwardsToggled(bool b) {
  setDate2();
  setDateButtons();
}

void QualendarFind::setDateButtons() {
  pbStart->setText(QString("%1.%2.%3").arg(date_.year()).arg(date_.month()).arg(date_.day()));
  pbEnd->setText(QString("%1.%2.%3").arg(date2.year()).arg(date2.month()).arg(date2.day()));
}

void QualendarFind::slotStartDate() {
  DatePickerDialog dlg(this, date_);
  dlg.exec();
  setDate(dlg.date());
}
void QualendarFind::slotEndDate() {
  DatePickerDialog dlg(this, date2);
  dlg.exec();
  QDate d = dlg.date();
  if ((cbBackwards->isChecked() && date_ >= d) ||
      (!cbBackwards->isChecked() && date_ <= d))
    date2 = d;
  setDateButtons();
}

void QualendarFind::slotEventSelected(QListViewItem *lvi) {
  if (!lvi)
    return;
  emit viewItem(QualendarItem(EffectiveEvent(((QualendarFindItem*)lvi)->event, ((QualendarFindItem*)lvi)->date)));
  emit dateChanged(((QualendarFindItem*)lvi)->date);
}

void QualendarFind::setDate2() {
  date2 = date_.addDays(cbBackwards->isChecked() ? -365 : 365);
}
