#include "qualendarmonth.h"
#include "../qualendarglobal.h"
#include "../qualendarcalendar.h"
#include "qualendarmonthheader.h"
#include "../sysdep/qualendardb.h"
#include <qpe/calendar.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qwidgetstack.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qpainter.h>
#include <qlistbox.h>
#include <qpe/resource.h>
#include <qpe/config.h>

#include "../debug.h"

QualendarMonth::QualendarMonth(QWidget *parent, const char *name, 
			       QArray<QualendarDB *> dataBases, 
			       QPopupMenu *menu) 
  : QualendarViewEx(parent, name, dataBases, menu), menu_(menu) {

  header = new QualendarMonthHeader(this);

  mainArea = new QualendarMainArea(this);
  mainArea->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,
				      QSizePolicy::Expanding));
  mainArea->setFocusPolicy(NoFocus);

  makeConnections();

  ag = new QActionGroup(this);
  ag->setExclusive(true);
  QAction *a;
  a = new QAction("both", tr("both"), 0, ag, 0, true);
  a->setOn(true);
  desc = true, graph = true, both2 = false, iconOnly = false;

  a = new QAction("simple", tr("simple"), 0, ag, 0, true);
  a = new QAction("text", tr("text only"), 0, ag, 0, true);
  a = new QAction("graph", tr("bar only"), 0, ag, 0, true);
  a = new QAction("icon", tr("icon only"), 0, ag, 0, true);
  connect(ag, SIGNAL(selected(QAction *)), this, SLOT(slotViewOption(QAction *)));
  dayBox = new QListBox(mainArea);
  dayBox->setHScrollBarMode(QScrollView::AlwaysOff);
  connect(dayBox, SIGNAL(returnPressed(QListBoxItem*)), 
	  this, SLOT(slotDayBox(QListBoxItem*)));
  connect(dayBox, SIGNAL(clicked(QListBoxItem*)), 
	  this, SLOT(slotDayBox(QListBoxItem*)));
}
QString QualendarMonth::viewName() const {
  return "Month";
}
QString QualendarMonth::trViewName() const {
  return tr("Month");
}
QPixmap QualendarMonth::icon() const {
  return Resource::loadPixmap("qualendar/month.png");
}
void QualendarMonth::setupMenu() {
  ag->addTo(menu_);
}

void QualendarMonth::keyPressEvent(QKeyEvent *e) {
  if (e->key() == Key_Escape && dayBox->hasFocus())
    mainArea->setFocus();
  else if (returnDayBox && e->key() == Key_Return) 
    dayBox->setFocus();
  else
    QualendarViewEx::keyPressEvent(e);
}

QDate QualendarMonth::dateStart() const {
  return QDate(date_.year(), date_.month(), 1);
}

QDate QualendarMonth::dateEnd() const {
  return QDate(date_.year(), date_.month(), date_.daysInMonth());
}

QDate QualendarMonth::tableDateStart() const {
  int y = date_.year(), m = date_.month();
  QDate d(y, m, 1);
  QValueList<QualendarCalendar::Day> days =
    QualendarCalendar::daysOfMonth(y, m, QualendarGlobal::onMonday);
  if (days[0].type != QualendarCalendar::Day::ThisMonth) {
    d = d.addDays(-1);
    d.setYMD(d.year(), d.month(), days[0].date);
  }
  return d;
}

QDate QualendarMonth::tableDateEnd() const {
  int y = date_.year(), m = date_.month();
  QDate d(y, m, date_.daysInMonth());
  QValueList<QualendarCalendar::Day> days =
    QualendarCalendar::daysOfMonth(y, m, QualendarGlobal::onMonday);
  if (days[days.count()-1].type != QualendarCalendar::Day::ThisMonth) {
    d = d.addDays(1);
    d.setYMD(d.year(), d.month(), days[days.count()-1].date);
  }
  return d;
}

QDate QualendarMonth::eventsDateStart() const {
  if (showOtherMonths) 
    return tableDateStart();
  else
    return dateStart();
}

QDate QualendarMonth::eventsDateEnd() const {
  if (showOtherMonths)
    return tableDateEnd();
  else
    return dateEnd();
}

void QualendarMonth::setupDateRects() {
  QDate dStart = tableDateStart();
  dateRects.resize(7, 6);

  for (uint i=0; i<42; i++)
    dateRects[i].setDate(dStart.addDays(i));
}

void QualendarMonth::loadConfig() {
  header->setFontSize(QualendarGlobal::fontSize);

  Config config("Qualendar");
  config.setGroup("Month");
  showOtherMonths = config.readBoolEntry("ShowOtherMonths", false);
  fontSize = config.readNumEntry("FontSize", QualendarGlobal::fontSize);
  if (fontSize < 0) 
    fontSize = font().pixelSize();
  dateFontSize = config.readNumEntry("DateFontSize", fontSize);
  todoStart = config.readBoolEntry("TodoStart", true);
  todoDue = config.readBoolEntry("TodoDue", true);
  jumpToItem = config.readBoolEntry("JumpToItem", true);
  returnDayBox = config.readBoolEntry("ReturnDayBox", false);

  eventsFirst = config.readBoolEntry("EventsFirst", false);
  dayBoxEventsFirst = config.readBoolEntry("DayBoxEventsFirst", false);
}

void QualendarMonth::relayout() {
  DRAW_DEBUG(qDebug("QualendarMonth::relayout"));
#ifdef DEBUG_PROFILE
  QTime tm;
  tm.start();
#endif
  QWidget *w = mainArea;
  if (fontSize >= 0)
    w->setFont(QFont(w->font().family(), fontSize));
  QPainter p(w);
  QFontMetrics fm = p.fontMetrics();

  headerHeight = fm.ascent() + 2;
  cellWidth = w->width()/7;
  cellHeight = (w->height() - headerHeight)/6;

  downPixmap = Resource::loadPixmap("qualendar/down").convertToImage()
    .smoothScale(fm.height(), fm.height()/2);

  // layout day cells
  for ( int l = 0; l < 6; ++l ) 
    for ( int k = 0; k < 7; ++k ) 
      dateRects(k, l).setRect(k*cellWidth, headerHeight + l*cellHeight,
			      cellWidth, cellHeight);

  // layout events
  itemRects.clear();
  if (desc) {
    for(int l = 0; l < 6; l++) {
      for(int k = 0; k < 7; k++) {
	DateRect &dr = dateRects(k, l);
	dr.tooMany = false;
	int ix = (iconOnly ? fm.height()*3/2 : 0), iy = 0;

	if (eventsFirst) {
	  iy = layoutEvents(dr, fm, ix, iy);
	  layoutTasks(dr, fm, ix, iy);
	} else {
	  iy = layoutTasks(dr, fm, ix, iy);
	  layoutEvents(dr, fm, ix, iy);
	}
      }
    }
  }

  QDate d0(date_.year(), date_.month(), 1);
  d0 = d0.addDays(d0.daysInMonth() - 1);
  int n = d0.daysTo(dateRects[41].date()) + 1;
  if (n > 6) 
    dayBox->setGeometry(cellWidth * 2, headerHeight + cellHeight * 5, 
			  cellWidth * 5, cellHeight);
  else
    dayBox->setGeometry(0, headerHeight, cellWidth * 5, cellHeight);

#ifdef DEBUG_PROFILE
  qDebug("QualendarMonth::relayout %d ms", tm.elapsed());
#endif
}

void QualendarMonth::draw(QRect, bool useDirty) {
#ifdef DEBUG_PROFILE
  QTime tm;
  tm.start();
#endif
  QWidget *w = mainArea;
  QPainter p(w);
  QFontMetrics fm = p.fontMetrics();

  bool onMonday = QualendarGlobal::onMonday;


  // background
  QRect updatedRect;
  if (!useDirty) {
    p.fillRect(0, 0, w->width(), headerHeight, QColor(255, 255, 255));
    p.fillRect(0, headerHeight + cellHeight * 6, 
	       w->width(), w->height() - (headerHeight + cellHeight * 6), 
	       QColor(255, 255, 255));
  }
  for (int l = 0; l < 6; l++) {
    for (int k = 0; k < 7; k++) {
      const DateRect &dr = dateRects(k, l);
      if (useDirty && !dr.dirty())
	continue;
      DRAW_DEBUG(qDebug("Draw background %d %d", k, l));
      QColor c;
      if (dr.date().month() != date_.month())
	c = QualendarGlobal::otherMonthColor;
      else
	c = getDayColor(dr.date());
      if (selectionMode &&
	  dr.date() >= selStart && dr.date() <= selEnd)
	p.fillRect(dr, QColor(0, 0, 255));
      else {
	p.fillRect(dr, c);
	if (!dr.background().isNull())
	  p.drawPixmap(dr.x() + dr.width() - dr.background().width() - 1, 
		       dr.y(), 
		       dr.background());
      }
      updatedRect |= dr;
    }
  }

  // grid
  p.drawLine(0, 0, w->width(), 0);
  for ( int y = 0; y <= 6; ++y )
    p.drawLine(0, headerHeight + y*cellHeight,
			   w->width(), headerHeight + y*cellHeight);
  for ( int x = 0; x <= 7; ++x )
    p.drawLine(x*cellWidth, 0, x*cellWidth, headerHeight + 6*cellHeight);

  // header
  if (!useDirty)
    for (uint i = 0; i < 7; ++i) {
      p.drawText(i*cellWidth + 1, headerHeight - 1,
		 dayName((i+ (onMonday ? 1:0))%7));
    }

  // graph

  if (graph) {
    for(int l = 0; l < 6; l++) {
      for(int k = 0; k < 7; k++) {
	if (useDirty && !dateRects(k, l).dirty())
	  continue;
	DRAW_DEBUG(qDebug("Draw graph %d %d", k, l));
	for(uint i=0; i < dateRects(k, l).events().count(); i++) {
	  const EffectiveEvent &ev = dateRects(k, l).events()[i];
	  const Event &e = ev.event();
	  int y;
	  QColor color;
	  if (dateRects(k, l).eventHidden(i))
	    continue;
	  if (e.type() == Event::AllDay) {
	    if (iconOnly)
	      y = cellHeight - 3;
	    else
	      y = 1;
	    color = QualendarGlobal::allDayColor;
	  } else {
	    if (iconOnly)
	      y = cellHeight - 6;
	    else
	      y = cellHeight / 2;
	    if (e.hasAlarm())
	      color = QualendarGlobal::alarmColor;
	    else
	      color = QualendarGlobal::eventColor;
	  }

	  if (dateRects(k, l).date().month() != date_.month()) 
	    color = QColor((color.red()+200)/2,
			   (color.green()+200)/2,
			   (color.blue()+200)/2);
	  int a = ev.start().hour()*60 + ev.start().minute();
	  int b = ev.end().hour()*60 + ev.end().minute();
	  int x0 = (cellWidth-2)*a/1440;
	  int x1 = (cellWidth-2)*(b-a)/1440+2;
	  p.fillRect(k * cellWidth + x0 + 1,
		     headerHeight + l*cellHeight + y,
		     x1, 3,
		     color);
	}
      }
    }
  }

  // event description
  QRect updatedRect1 = updatedRect;
  if (desc) {
    p.setClipping(true);
    p.setClipRect(0, 0, w->width(), w->height());
    for(uint i = 0; i < itemRects.count(); i++) {
      ItemRect er = itemRects[i];
      if (er.intersects(updatedRect)) {
	DRAW_DEBUG(qDebug("Draw event %d", i));
	er.draw(p);
	updatedRect1 |= er;
      }
    }
    p.setClipping(false);
  }

  // day number
  bool dayOnBottomRight = iconOnly;

  p.setFont(QFont(p.font().family(), dateFontSize));
  fm = p.fontMetrics();
  for ( int l = 0; l < 6; l++ ) {
    for ( int k = 0; k < 7; k++ ) {
      const DateRect &dr = dateRects(k, l);
      if (useDirty && !dr.intersects(updatedRect1))
	continue;
      int dy = dr.date().day();
      int xOffset = dayOnBottomRight ? 
	2 : dr.width() - (dy<10 ? fm.ascent()/2 : fm.ascent());
      int yOffset = dayOnBottomRight ? fm.ascent() : dr.height();
      p.fillRect(dr.x() + xOffset - 2,
		 dr.y() + yOffset - fm.ascent(), 
		 (dy < 10 ? fm.ascent()/2 : fm.ascent()) + 2, 
		 fm.ascent()+1, 
		 (dr.date().month() == date_.month()) ? 
		 QColor(255, 255, 255) : QualendarGlobal::otherMonthColor);
      p.setPen(QColor(0, 0, 0));
      p.drawRect(dr.x() + xOffset - 2,
		 dr.y() +yOffset - fm.ascent(), 
		 (dy < 10 ? fm.ascent()/2 : fm.ascent()) + 3, 
		 fm.ascent()+1);
      QDate d = dr.date();
      p.setPen(QColor(0, 0, 0));
      if (!QualendarGlobal::colorOnBG) {
	if ((d.dayOfWeek())%7 == 0 || QualendarGlobal::holidays.contains(d)) {
	  p.setPen(QColor(255, 0, 0));
	} else if ((d.dayOfWeek()) == 6) {
	  p.setPen(QColor(0, 0, 255));
	}
      }
      p.drawText(dr.x() + xOffset - 1, 
		 dr.y() + yOffset, 
		 QString::number(d.day()));
      if (dr.tooMany) 
	p.drawPixmap(dr.x() + (dr.width() - downPixmap.width())/2, 
		     dr.y() + dr.height() - downPixmap.height(), 
		     downPixmap);
    }
  }

  DRAW_DEBUG(qDebug("highlightDate " + viewName() + "%d %d %d", date_.year(), date_.month(), date_.day()));
  p.setPen(QColor(0, 0, 0));
  int i, j;
  if (findDate(date_, i, j)) {
    QRect r = dateRects(i, j);
    p.drawRect(r.x()+1, r.y()+1, r.width()-1, r.height()-1);
  }

  for(int l = 0; l < 6; l++) 
    for(int k = 0; k < 7; k++) 
      dateRects(k, l).setDirty(false);


  // day box.
  dayBox->clear();
  dayItems.clear();

  for(uint rep = 0; rep < 2; rep++) {
    if ((rep == 0 && !dayBoxEventsFirst) || (rep == 1 && dayBoxEventsFirst)) {
      // tasks.
      tasks = getTasks(date_, date_, filter_);
      for(uint i=0; i < tasks.count(); i++) {
	QString s = QString("%1 ").arg(tasks[i].priority());
	QString t = dateString(tasks[i], true, true);
	if (!t.isEmpty())
	  s += "(" + t + ") ";
	dayBox->insertItem(s + tasks[i].description());
	dayItems.append(QualendarItem(tasks[i]));
      }

    } else {
      // events.
      int k, l;
      findDate(date_, k, l);

      for(uint i = 0; i < dateRects(k, l).events().count(); i++) {
	const EffectiveEvent &e = dateRects(k, l).events()[i];
	QString s;
	if (e.event().type() == Event::AllDay) 
	  s += "===== ";
	else
	  s += timeString(e) + " ";
	s += e.description();
	if (!e.location().isEmpty())
	  s += QString(" (%1)").arg(e.location());

	dayBox->insertItem(s);
	dayItems.append(QualendarItem(e));
      }
    }
  }

#ifdef DEBUG_PROFILE
  qDebug("QualendarMonth::draw, useDirty=%d, %d ms", useDirty, tm.elapsed());
#endif
}

QualendarItem QualendarMonth::selectedItem() {
  int k, l;
  int i = dayBox->currentItem();
  if (dayBox->hasFocus() && i >= 0) {
    return dayItems[i];
#if 0
  } else if (findDate(date_, k, l)) {
    if (dateRects(k, l).events().count() > 0)
      return dateRects(k, l).events()[0].event();
#endif
  }
  return QualendarItem();
}

void QualendarMonth::slotViewOption(QAction *a) {
  QString t = a->text();
  if (t == "both") {
    desc = true, graph = true, both2 = false, iconOnly = false;
  } else if (t == "simple") {
    desc = true, graph = true, both2 = true, iconOnly = false;
  } else if (t == "text") {
    desc = true, graph = false, both2 = false, iconOnly = false;
  } else if (t == "graph") {
    desc = false, graph = true, both2 = false, iconOnly = false;
  } else if (t == "icon") {
    desc = true, graph = true, both2 = true, iconOnly = true;
  }
  redraw();
}

void QualendarMonth::slotDayBox(QListBoxItem* lbi) {
  if (!lbi)
    return;
  int i = dayBox->index(lbi);
  emit viewItem(dayItems[i]);
}

int QualendarMonth::layoutEvents(DateRect &dr, const QFontMetrics &fm, 
				 int ix, int iy) {
  int ox = dr.x(), oy = dr.y();
  QDate d = dr.date();

  for(uint i = 0; i < dr.events().count(); i++) {
    if (iy >= cellHeight - fm.height()*2/3)
      dr.tooMany = true;

    ItemRect er;

    QDateTime dt;
    const EffectiveEvent &eev = dr.events()[i];
    const Event &ev = eev.event();
    QPixmap pm = QualendarGlobal::getBackground(ev.description(), cellWidth, cellHeight);
    if (!pm.isNull()) {
      if (dr.date().month() == date_.month())
	dr.setBackground(pm);
      dr.setEventHidden(i, true);
      continue;
    } else
      dr.setEventHidden(i, false);

    if (iy >= cellHeight)
      continue;

    bool hasNext = true;
    if (ev.repeatPattern().type == Event::NoRepeat)
      dt = ev.start();
    else 
      hasNext = db->nextOccurance(ev, d, dt);

    if (!both2 || d.day() == 1 || d == dt.date() || !hasNext) {
      QColor color = getColor(ev);
      if (d.month() != date_.month())
	color = QColor((color.red()+200)/2, (color.green()+200)/2, (color.blue()+200)/2);
      QString s = ev.description();
      QPixmap pm;

      s = getIcon(s, fontSize, pm);
      if (iconOnly) { // icon-only mode
	if (!pm.isNull()) {
	  if (ix + pm.width() > cellWidth + pm.width()/3) {
	    ix = 0;
	    iy += pm.height();
	  }
	  er.moveTopLeft(QPoint(ox+ix, oy+iy));
	  er.addPixmap(pm);
	  ix += pm.width() + 1;
	} else if (!s.isEmpty()) {
	  if (ix + fm.width(s.left(1)) > cellWidth + fm.width(s.left(1))/3) {
	    ix = 0;
	    iy += fm.height();
	  }
	  er.moveTopLeft(QPoint(ox+ix, oy+iy));
	  er.addText(fm, s.left(1), color);
	  ix += fm.width(s.left(1)) + 1;
	}
	er.setItem(eev);
	er.clip(dr);
	itemRects.append(er);
      } else { // one event per line
	er.moveTopLeft(QPoint(ox+ix, oy+iy));
	if (!pm.isNull()) {	
	  er.addPixmap(pm);
	  ix += pm.width() + 1;
	}
	if (!s.isEmpty()) {
	  er.addText(fm, s, color);
	}
	ix = 0;
	iy += fm.height();
	er.setItem(eev);
	er.clip(dr);
	itemRects.append(er);
      }
    }
  }
  return iy;
}

int QualendarMonth::layoutTasks(DateRect &dr, const QFontMetrics &fm, 
				 int ix, int iy) {
  int ox = dr.x(), oy = dr.y();
  QDate d = dr.date();

  for(uint i = 0; i < dr.tasks().count(); i++) {
    if (iy >= cellHeight - fm.height()*2/3)
      dr.tooMany = true;
    if (iy >= cellHeight) 
      continue;

    ItemRect er;

    const QualendarTask &task = dr.tasks()[i];
    QColor color = getTodoColor(task, dr.date());
    QPixmap todoPM = getTodoPixmap(task, dr.date(), fontSize);
    QPixmap pm;
    if (d.month() != date_.month())
      color = QColor((color.red()+200)/2, (color.green()+200)/2, (color.blue()+200)/2);

    QString s = task.description();

    s = getIcon(s, fontSize, pm);
    if (iconOnly) { // icon-only mode
      if (!pm.isNull()) {
	if (ix + todoPM.width() + pm.width() 
	    > cellWidth + pm.width()/3) {
	  ix = 0;
	  iy += pm.height();
	}
	er.moveTopLeft(QPoint(ox+ix, oy+iy));
	er.addPixmap(todoPM);
	er.addPixmap(pm);
	ix += todoPM.width() + pm.width() + 2;
      } else if (!s.isEmpty()) {
	if (ix + todoPM.width() + fm.width(s.left(1)) 
	    > cellWidth + fm.width(s.left(1))/3) {
	  ix = 0;
	  iy += fm.height();
	}
	er.moveTopLeft(QPoint(ox+ix, oy+iy));
	er.addPixmap(todoPM);
	er.addText(fm, s.left(1), color);
	ix += todoPM.width() + fm.width(s.left(1)) + 2;
      }
      er.setItem(task);
      er.clip(dr);
      itemRects.append(er);
    } else { // one task per line
      er.moveTopLeft(QPoint(ox+ix, oy+iy));
      er.addPixmap(todoPM);
      ix += todoPM.width() + 1;
      if (!pm.isNull()) {	
	er.addPixmap(pm);
	ix += pm.width() + 1;
      }
      if (!s.isEmpty()) 
	er.addText(fm, s, color);
      ix = 0;
      iy += fm.height();
      er.setItem(task);
      er.clip(dr);
      itemRects.append(er);
    }
  }
  return iy;
}
