#include "qualendartodoview.h"
#include "../qualendaritem.h"
#include "../qualendarglobal.h"
#include "../utils/datepicker.h"
#include "../sysdep/qualendardb.h"

#include <qtl.h>
#include <qframe.h>
#include <qpopupmenu.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qaction.h>

#include <qpe/calendar.h>
#include <qpe/resource.h>
#include <qpe/config.h>

#include "../debug.h"

class QualendarTodoViewItem : public QListViewItem {
 public:
  QualendarTodoViewItem(QListView *parent, QualendarTask t, 
			const QMap<QString, int> &nameToColumn) 
    : QListViewItem(parent), task(t), nameToColumn_(nameToColumn) {
    if (nameToColumn.contains("Done") && t.isCompleted())
      setText(nameToColumn["Done"], "x");
    if (nameToColumn.contains("Priority"))
      setText(nameToColumn["Priority"], QString::number(t.priority()));
    if (nameToColumn.contains("Description"))
      setText(nameToColumn["Description"], t.description());
    if (nameToColumn.contains("Start") && t.hasStartDate()) {
      QDate d = t.startDate();
      setText(nameToColumn["Start"], QString("%1/%2/%3")
	      .arg(d.year()%100, 2)
	      .arg(d.month(), 2)
	      .arg(d.day(), 2)
	      .replace(QRegExp(" "), "0"));
    }
    if (nameToColumn.contains("Due") && t.hasDueDate()) {
      QDate d = t.dueDate();
      setText(nameToColumn["Due"], QString("%1/%2/%3")
	      .arg(d.year()%100, 2)
	      .arg(d.month(), 2)
	      .arg(d.day(), 2)
	      .replace(QRegExp(" "), "0"));
    }
    if (nameToColumn.contains("Finish") && t.isCompleted()) {
      QDate d = t.finishDate();
      setText(nameToColumn["Finish"], QString("%1/%2/%3")
	      .arg(d.year()%100, 2)
	      .arg(d.month(), 2)
	      .arg(d.day(), 2)
	      .replace(QRegExp(" "), "0"));
    }
  }
  QString key(int column, bool ascending) const {
    QDate d;
    QMapConstIterator<QString, int> it;
    if ((it = nameToColumn_.find("Start")) != nameToColumn_.end() &&
	it.data() == column)
      d = task.startDate();
    if ((it = nameToColumn_.find("Due")) != nameToColumn_.end() &&
	it.data() == column)
      d = task.dueDate();
    if ((it = nameToColumn_.find("Finish")) != nameToColumn_.end() &&
	it.data() == column)
      d = task.finishDate();
    if (d.isValid()) {
      QString s = "000000";
      QString t = QString::number(QDate(1970, 1, 1).daysTo(d));
      return s.left(6 - t.length()) + t; // 5 should be ok...
    } else {
      return text(column);
    }
  }
  QualendarTask task;
 private:
  const QMap<QString, int> &nameToColumn_;
};

QualendarTodoListView::QualendarTodoListView(QWidget *parent)
  : QListView(parent) {
}

void QualendarTodoListView::keyPressEvent(QKeyEvent *e) {
  if (e->key() == Key_Space)
    emit spacePressed(currentItem());
  else if ((e->ascii() >= '0' && e->ascii() <= '9') || 
	   (e->ascii() >= 'a' && e->ascii() <= 'z') || 
	   e->key() == Key_Delete)
    e->ignore();
  else
    QListView::keyPressEvent(e);
}

QualendarTodoView::QualendarTodoView(QWidget *parent, const char *name, 
				     QArray<QualendarDB *> dataBases, 
				     QPopupMenu *menu) 
  : QualendarView(parent, name, dataBases, menu), menu_(menu), savedUid(0) {

  QFrame *f;
  QHBoxLayout *lo;

  f = new QFrame(this);
  lo = new QHBoxLayout(f);

  lvResult = new QualendarTodoListView(this);

  lvResult->setSorting(1);

  menu->clear();

  connect(lvResult, SIGNAL(clicked(QListViewItem *, const QPoint &, int)),
	  this, SLOT(slotClicked(QListViewItem *, const QPoint &, int)));
  connect(lvResult, SIGNAL(returnPressed(QListViewItem *)), 
	  this, SLOT(slotTaskSelected(QListViewItem *)));
  connect(lvResult, SIGNAL(spacePressed(QListViewItem *)), 
	  this, SLOT(slotTaskDone(QListViewItem *)));

  actionShowDone = new QAction("", tr("Completed tasks"), 
			       0, this, 0, true);
  actionShowNotDone = new QAction("", tr("Incomplete tasks"), 
				  0, this, 0, true);
  actionShowDone->setOn(true);
  connect(actionShowDone, SIGNAL(activated()),
          this, SLOT(redraw()));
  actionShowNotDone->setOn(true);
  connect(actionShowNotDone, SIGNAL(activated()),
          this, SLOT(redraw()));

  redraw();
}

QualendarTodoView::~QualendarTodoView() {
  Config tmpConfig("QualendarTmp");
  tmpConfig.setGroup("TodoView");

  QMapConstIterator<QString, int> it;
  for(it = nameToColumn.begin(); it != nameToColumn.end(); ++it) 
    tmpConfig.writeEntry(it.key(), lvResult->columnWidth(it.data()));
}

QString QualendarTodoView::viewName() const {
  return "Todo";
}
QString QualendarTodoView::trViewName() const {
  return tr("Todo");
}
QPixmap QualendarTodoView::icon() const {
  return Resource::loadPixmap("qualendar/check.png");
}
QDate QualendarTodoView::currentDate() const {
  return date_;
}
void QualendarTodoView::selectedDateTime(QDateTime &start, QDateTime &end) const {
  start = QDateTime(date_, QTime(10, 0, 0));
  end = QDateTime(date_, QTime(12, 0, 0));
}
void QualendarTodoView::setupMenu() {
  actionShowDone->addTo(menu_);
  actionShowNotDone->addTo(menu_);
}

void QualendarTodoView::keyPressEvent(QKeyEvent *e) {
  if (e->key() == 0x201a) { // Fn + M
    emit editItem(selectedItem());
  } else if (e->key() == Key_Delete) {
    emit removeItem(selectedItem());
  } else if (e->key() == Key_C) {
    actionShowDone->setOn(!actionShowDone->isOn());
    redraw();
  } else if (e->key() == Key_I) {
    actionShowNotDone->setOn(!actionShowNotDone->isOn());
    redraw();
  } else {
    QualendarView::keyPressEvent(e);
  }
}

QualendarItem QualendarTodoView::selectedItem() {
  QualendarTodoViewItem *lvi = (QualendarTodoViewItem *)(lvResult->currentItem());
  if (lvi) 
    return QualendarItem(lvi->task);
  else
    return QualendarItem();
}

void QualendarTodoView::slotTaskSelected(QListViewItem *lvi) {
  if (!lvi)
    return;

  QualendarTodoViewItem *qtvi = (QualendarTodoViewItem *) lvi;
  emit viewItem(QualendarItem(qtvi->task));
}

void QualendarTodoView::slotTaskDone(QListViewItem *lvi) {
  if (!lvi)
    return;

  QualendarTodoViewItem *qtvi = (QualendarTodoViewItem *) lvi;
  QualendarTask t = qtvi->task;
  bool b = !t.isCompleted();
  t.setCompleted(b);
  t.setFinishDate(QDate::currentDate());
  savedUid = qtvi->task.uid();
  taskDB->edit(qtvi->task, t);
}

void QualendarTodoView::slotClicked(QListViewItem *lvi, const QPoint &, int i) {
  if (nameToColumn.contains("Done") && 
      i == nameToColumn["Done"])
    slotTaskDone(lvi);
  else
    slotTaskSelected(lvi);
}

void QualendarTodoView::setFilter(QMap<int, bool> filter, bool draw) {
  filter_ = filter;
  if (draw)
    redraw();
}

void QualendarTodoView::setDate(const QDate &d, bool mustDraw) {
  date_ = d;
}

void QualendarTodoView::loadConfig() {
  Config config("Qualendar");
  config.setGroup("TodoView");
  fontSize = config.readNumEntry("FontSize", QualendarGlobal::fontSize);
  if (fontSize < 0) 
    fontSize = font().pixelSize();
  else {
    QFont f(font().family(), fontSize);
    setFont(f);
  }


  Config tmpConfig("QualendarTmp");
  tmpConfig.setGroup("TodoView");

  lvResult->clear();
  while(lvResult->columns() > 0)
    lvResult->removeColumn(0);
  int w = fontMetrics().width('m');

  struct {
    char *name; 
    QString title;
    int width; 
  } data[6] = {
    { "Done", tr("Done"), 4 },
    { "Priority", tr("P"), 4 }, 
    { "Description", tr("Description"), 20 }, 
    { "Start", tr("Start"), 8 }, 
    { "Due", tr("Due"), 8 }, 
    { "Finish", tr("Finish"), 8 }
  };

  uint col = 0;
  for(uint i=0; i<6; i++) {
    QString colName = config.readEntry("Column" + QString::number(i), 
				       data[i].name);
    uint j;
    for(j=0; j<6; j++) {
      if (colName == data[j].name)
	break;
    }
    if (j < 6) {
      int columnWidth = tmpConfig.readNumEntry(data[j].name, (data[j].width + 1) * w);
      lvResult->addColumn(data[j].title, columnWidth);
      nameToColumn.insert(colName, col);
      col++;
    }
  }
}

void QualendarTodoView::redraw() {
  lvResult->clear();
  const QValueList<QualendarTask> &tasks = filterTasks(taskDB->getTasks(), filter_);

  QValueListConstIterator<QualendarTask> it;
  for (it = tasks.begin(); it != tasks.end(); ++it) {
    if (actionShowDone->isOn() && (*it).isCompleted() ||
	actionShowNotDone->isOn() && !(*it).isCompleted()) {
      QualendarTodoViewItem *lvi = 
	new QualendarTodoViewItem(lvResult, *it, nameToColumn);
      if ((*it).uid() == savedUid)
	lvResult->setCurrentItem(lvi);
    }
  }
}
