/***************************************************************************
                          datebook2viewex.cpp  -  description
                             -------------------
    begin                : 2003.8.29
    copyright            : (C) 2003 by ichitokumei
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qscrollview.h>
#include <qhbox.h>
#include <qapplication.h>

#include "qualendarviewex.h"
#include "../qualendarglobal.h"
#include "../sysdep/qualendardb.h"

#include "../debug.h"

QualendarViewHeader::QualendarViewHeader(QWidget *parent)
  : QWidget(parent) {
  setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
}

/////////////////////////////////////////////////////
QualendarViewEx::QualendarViewEx(QWidget *parent, const char *name,
				 QArray<QualendarDB *> dataBases, 
				 QPopupMenu *viewMenu)
  : QualendarView(parent, name, dataBases, viewMenu), 
    todoStart(true), todoDue(true), jumpToItem(true) {
  drawReady = false;
  tableChanged = true;
  mustRelayout = true;
  selectionMode = false;
  selEnd = selStart.addDays(-1); // for safety;
}

void QualendarViewEx::makeConnections() {
  connect(mainArea, SIGNAL(paintMe(QRect)),
	  this, SLOT(paintMainArea(QRect)));
  connect(mainArea, SIGNAL(mouseEvent(QMouseEvent *)),
	  this, SLOT(slotMouseEvent(QMouseEvent *)));
  connect(header, SIGNAL(dateChanged(const QDate &)),
	  this, SLOT(setDate(const QDate &)));
}

/**
  Gives the selected DateTime
*/
void QualendarViewEx::selectedDateTime(QDateTime &start, QDateTime &end) const {
  start = selStart;
  end = selEnd;
  return;
}

/*!
  Sets the date to \a d.
 */
void QualendarViewEx::setDate(const QDate &d, bool mustDraw) {
  DRAW_DEBUG(qDebug("setDate " + viewName() + ": %d %d %d", d.year(), d.month(), d.day()));

  bool outOfRange = (!date_.isValid() || d < dateStart() || d > dateEnd());

  QDate oldDate = date_;
  date_ = d;
  selectionMode = false;
  selStart = date_;
  selEnd = date_;

  header->setDate(date_);

  if (outOfRange) {
    tableChanged = true;
    mustRelayout = true;
  }

  if (!mustDraw)
    return;

  if (tableChanged) {
    redraw(); // it can happen that paintMainArea is not immediately called.
  } else {
    for (int l = 0; l < dateRects.h(); l++)
      for (int k = 0; k < dateRects.w(); k++)
	if (dateRects(k, l).date() == oldDate)
	  dateRects(k, l).setDirty(true);
    draw(QRect(0, 0, mainArea->width(), mainArea->height()), true);
  }
}
void QualendarViewEx::setDate(const QDate &d) {
  setDate(d, true);
}

void QualendarViewEx::setFilter(QMap<int, bool> filter, bool draw) {
  filter_ = filter;
  tableChanged = true;
  mustRelayout = true;
  if (draw) {
    DRAW_DEBUG(qDebug("Draw from setFilter"));
    redraw();
  }
}

void QualendarViewEx::redraw() {
  if (!drawReady)
    return;
  tableChanged = true;
  mustRelayout = true;
  DRAW_DEBUG(qDebug("Draw from redraw " + viewName()));
  mainArea->update();
}

void QualendarViewEx::paintMainArea(QRect r) {
  if (!date_.isValid())
    return;
  DRAW_DEBUG(qDebug("paintMainArea() " + viewName() +
		    ", drawReady=%d, tableChanged=%d, mustRelayout=%d",
		    drawReady, tableChanged, mustRelayout));
  //  loadConfig();
  // should relayout if !clean
  if (drawReady) {
    if (tableChanged) {
      setupDateRects();
      getEvents();
      tableChanged = false;
    }
    if (mustRelayout) {
      relayout();
      mustRelayout = false;
    }
    draw(r);
  }
}

void QualendarViewEx::resizeEvent(QResizeEvent *e) {
  DRAW_DEBUG(qDebug("resizeEvent %d %d", width(), height()));
  QWidget *d = QApplication::desktop();
  if (width() < d->width()/2) {
    drawReady = false;
  } else {
    drawReady = true;
    mustRelayout = true;
  }
}

void QualendarViewEx::keyPressEvent(QKeyEvent *e) {
  INPUT_DEBUG(qDebug("QualendarViewEx::keyPressEvent %d %c", e->key(), e->ascii()));
  switch(e->key()) {
  case Key_Up:
    setDate(date_.addDays(-stepDown()));
    break;
  case Key_Down:
    setDate(date_.addDays(stepDown()));
    break;
  case Key_Left:
    setDate(date_.addDays(-stepRight()));
    break;
  case Key_Right:
    setDate(date_.addDays(stepRight()));
    break;
#ifndef VIEWONLY
  case Key_E:
  case 0x201a: // Fn + M
    emit editItem(selectedItem());
    break;
  case Key_Delete:
    emit removeItem(selectedItem());
    break;
#endif
  case Key_V:
    emit viewItem(selectedItem());
    break;
  default:
    e->ignore();
  }
}

bool QualendarViewEx::pointToEvent(QPoint p, int &a) {
  for(uint i = 0; i < itemRects.count(); i++) {
    if (itemRects[i].contains(p)) {
      a = i;
      return true;
    }
  }
  a = -1;
  return false;
}

bool QualendarViewEx::pointToCell(QPoint p, int &i1, int &j1) {
  for(int l = 0; l < dateRects.h(); l++) {
    for(int k = 0; k < dateRects.w(); k++) {
      if (dateRects(k, l).contains(p) && dateRects(k, l).date().isValid()) {
	i1 = k, j1 = l;
	return true;
      }
    }
  }
  return false;
}

void QualendarViewEx::myMouseEvent(QMouseEvent *e) {
  setFocus();
  int a, i, j;
  switch(e->type()) {
  case QEvent::MouseButtonPress:
    if (pointToEvent(e->pos(), a)) {
      selectedItemRect = a;
    } else {
      selectedItemRect = -1;
    }
    if (pointToCell(e->pos(), i, j)) {
      selX0 = i;
      selY0 = j;
      selX1 = i;
      selY1 = j;
      selStart = dateRects(selX0, selY0).date();
      selEnd = selStart.addDays(-1);
      selectionMode = true;
    }
    break;
  case QEvent::MouseMove:
    if (selectionMode) {
      if (pointToCell(e->pos(), i, j)) {
	if (selX1 != i || selY1 != j) {
	  QDate selStart0 = selStart, selEnd0 = selEnd;
	  selX1 = i;
	  selY1 = j;
	  QDate d0, d1;
	  d0 = dateRects(selX0, selY0).date();
	  d1 = dateRects(selX1, selY1).date();
	  selStart = QMIN(d0, d1);
	  selEnd = QMAX(d0, d1);
	  for (int l = 0; l < dateRects.h(); l++)
	    for (int k = 0; k < dateRects.w(); k++)
	      if ((dateRects(k, l).date() >= selStart0 && dateRects(k, l).date() <= selEnd0) !=
		  (dateRects(k, l).date() >= selStart && dateRects(k, l).date() <= selEnd))
		dateRects(k, l).setDirty(true);
	  draw(QRect(0, 0, mainArea->width(), mainArea->height()), true);
	}
      } else {
	selectionMode = false;
	for (int l = 0; l < dateRects.h(); l++)
	  for (int k = 0; k < dateRects.w(); k++)
	    if (dateRects(k, l).date() >= selStart && dateRects(k, l).date() <= selEnd)
	      dateRects(k, l).setDirty(true);
	draw(QRect(0, 0, mainArea->width(), mainArea->height()), true);
      }
    }
    break;
  case QEvent::MouseButtonRelease:
    if (selectionMode) {
      selectionMode = false;
      if (pointToCell(e->pos(), i, j) && (selX0 != i || selY0 != j)) {
	emit createEvent(QDateTime(selStart, QTime(10, 0, 0)), QDateTime(selEnd, QTime(12, 0, 0)), "");
      } else {
	for (int l = 0; l < dateRects.h(); l++)
	  for (int k = 0; k < dateRects.w(); k++)
	    if (dateRects(k, l).date() >= selStart && dateRects(k, l).date() <= selEnd)
	      dateRects(k, l).setDirty(true);
	if (selectedItemRect >= 0 && jumpToItem) 
	  emit viewItem(itemRects[selectedItemRect].item());
	else
	  setDate(dateRects(selX0, selY0).date());
      }
    }
    break;
  default:
    break;
  }
}

void QualendarViewEx::slotMouseEvent(QMouseEvent *e) {
  myMouseEvent(e);
}

void QualendarViewEx::getEvents() {
  DRAW_DEBUG(qDebug("getEvents: " + viewName()));
  if (!db)
    return;

  QDate dStart = eventsDateStart();
  QDate dEnd = eventsDateEnd();
#ifdef DEBUG_PROFILE
  QTime tm;
  tm.start();
#endif
  QValueList<EffectiveEvent> eList = db->getEffectiveEvents(dStart, dEnd);
#ifdef DEBUG_PROFILE
  qDebug("getEvents %d ms", tm.elapsed());
#endif
  for (int l = 0; l < dateRects.h(); l++)
    for (int k = 0; k < dateRects.w(); k++)
      dateRects(k, l).clear();
  for(QValueList<EffectiveEvent>::Iterator it = eList.begin();
      it != eList.end();
      ++it) {
    int i = ((*it).event().categories().count() > 0) ? 
      (*it).event().categories()[0] : QualendarGlobal::unfiledCatId;
    if (!filter_.contains(i)) {
      int k, l;
      if (findDate((*it).date(), k, l))
	dateRects(k, l).addEvent(*it);
    }
  }

  // construct the list of tasks.
  QValueList<QualendarTask> tl = filterTasks(taskDB->getTasks(), filter_);

  for(QValueListConstIterator<QualendarTask> it = tl.begin();
      it != tl.end();
      ++it) {
    const QualendarTask &task = *it;
    if (task.isCompleted())
      continue;
    int i = (task.categories().count() > 0) ? 
      task.categories()[0] : QualendarGlobal::unfiledCatId;
    //    if (filter_.contains(i)) 
    //      break;
    if (todoStart && task.hasStartDate()) {
      QDate d = task.startDate();
      if (!(todoDue && task.hasDueDate() && d == task.dueDate()) && 
	  d >= eventsDateStart() && d <= eventsDateEnd()) {
	int k, l;
	if (findDate(d, k, l)) 
	  dateRects(k, l).addTask(task);
      }
    } 
    if (todoDue && task.hasDueDate()) {
      QDate d = task.dueDate();
      if (d >= eventsDateStart() && d <= eventsDateEnd()) {
	int k, l;
	if (findDate(d, k, l))
	  dateRects(k, l).addTask(task);
      }
    }
  }
#ifdef DEBUG_PROFILE
  qDebug("getEvents--filter, %d ms", tm.elapsed());
#endif
}

bool QualendarViewEx::findDate(QDate d, int &i1, int &j1) {
  if (d < tableDateStart() || d > tableDateEnd())
    return false;
  for (int l = 0; l < dateRects.h(); l++) {
    for (int k = 0; k < dateRects.w(); k++) {
      if (dateRects(k, l).date() == d) {
	i1 = k, j1 = l;
	return true;
      }
    }
  }
  return false;
}

QDate QualendarViewEx::tableDateStart() const {
  return dateStart();
}
QDate QualendarViewEx::tableDateEnd() const {
  return dateEnd();
}
QDate QualendarViewEx::eventsDateStart() const {
  return dateStart();
}
QDate QualendarViewEx::eventsDateEnd() const {
  return dateEnd();
}
