/***************************************************************************
                          qualendarviewex.h  -  description
                             -------------------
    begin                : 2003.8.29
    copyright            : (C) 2003 by ichitokumei
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QUALENDARVIEWEX_H
#define QUALENDARVIEWEX_H

#include <qpixmap.h>
#include <qvector.h>
#include "../qualendarview.h"
#include "qualendarviewutil.h"

class QualendarItem;
class QualendarViewHeader;

/**
 * The abstract base class for QualendarMonth, QualendarWeek and QualendarDay.
 *
 * The widget has (at least) two children -- a header(to navigate through days)
 * and the main area.
 *
 * The interface for the header is defined in QualendarViewHeader.
 *
 * The main area(QualendarMainArea *mainArea) is just a QWidget 
 * which passes QPaintEvent and QMouseEvent (by signals).
 *
 * In a derived class, one has to create header and mainArea,
 * and call makeConnections.
 *
 * Drawing is done in 4 steps: 
 * setupDateRects(), getEvents(), relayout() and draw().
 *
 * setupDateRects() has to set up informations on dates 
 * in dateRects(see DateRect).
 *
 * getEvents() should also be OK for common usage:
 * the EffectiveEvent's are stored in dateRects.
 *
 *You have to implement relayout() and draw().
 **/

class QualendarViewEx : public QualendarView {
   Q_OBJECT
 public:
  QualendarViewEx(QWidget *parent, const char *name,
		  QArray<QualendarDB *>, QPopupMenu *);
  QDate currentDate() const {return date_;}
  void selectedDateTime(QDateTime &start, QDateTime &end) const;

 public slots:
  void setFilter(QMap<int, bool>, bool mustDraw = true);
  void setDate(const QDate &);
  void setDate(const QDate &, bool mustDraw);
  void redraw();
  void paintMainArea(QRect r);

 protected:
  void resizeEvent(QResizeEvent *);
  void keyPressEvent(QKeyEvent *);
  virtual void myMouseEvent(QMouseEvent *);
  bool todoStart, todoDue;
  bool jumpToItem;
 private slots:
  void slotMouseEvent(QMouseEvent *);

 private:
  void getEvents();

 protected:
  virtual QDate dateStart() const = 0;
  virtual QDate dateEnd() const = 0;
  virtual QDate tableDateStart() const;
  virtual QDate tableDateEnd() const;
  virtual QDate eventsDateStart() const;
  virtual QDate eventsDateEnd() const;
  virtual int stepRight() = 0;
  virtual int stepDown() = 0;

  /**
   * This is called when date went out of the current range
   * (determined by tableDateStart, tableDateEnd).
   * You have to resize dateRects 
   * and do dateRects[i].setDate(...) appropriately.
   **/
  virtual void setupDateRects() = 0;
  /**
   * This is called when the date changed and the events changed,
   * the widget was resized or redraw() was called.
   * Events may have changed after last call.
  **/
  virtual void relayout() = 0;
  /**
   * draw() has to draw the contents to mainArea(and others)
   * using dateRects and itemRects, which should be pre-calculated.
   *
   * This method is called when mainArea received a QPaintEvent
   * (including the case of mainArea->update() from redraw() etc.)
   * or the current date or selection changed.
   *
   * If useDirty is true, DateRect::dirty() can be used.
   * If selectionMode is true, dates between selStart and selEnd
   * should be displayed as such.
   * date_ is the current date, and it should be highlighted.
   **/
  virtual void draw(QRect r, bool useDirty = false) = 0;

  virtual QualendarItem selectedItem() = 0;

  void makeConnections();
  bool pointToEvent(QPoint, int &);
  bool pointToCell(QPoint, int &, int &);

  /* helper functions */
  bool findDate(QDate, int &, int &);

  QualendarViewHeader *header;

  QWidget *mainArea;

  QDate date_;
  QMap<int, bool> filter_;
  QMap<int, QColor> colors_;

  bool selectionMode;
  QDate selStart, selEnd;
  int selectedItemRect;
  int selX0, selY0, selX1, selY1;

  DateRectArray dateRects;
  QValueList<ItemRect> itemRects;
  QValueList<QualendarTask> taskList;

  bool drawReady, tableChanged, mustRelayout;
};

/**
	The base class for the header widget in QualendarViewEx.
 
	A class that inherits this class is supposed to have spinboxes, buttons etc.
	to change date.

	setDate() must be implemented.
	When the user operates on controls, it has to emit dateChanged().
	Take care not to emit signals after setDate().
 */

class QualendarViewHeader : public QWidget {
  Q_OBJECT
 public:
  QualendarViewHeader(QWidget *);
  virtual void setDate(const QDate &) = 0;
  virtual void setFontSize(int) {}
 signals:
  void dateChanged(const QDate &);
 protected:
  QDate date_;
};


#endif // QUALENDARVIEWEX_H
