#include "qualendarweek.h"
#include "../qualendaritem.h"
#include "../qualendarglobal.h"
#include "qualendarweekheader.h"
#include "../sysdep/qualendardb.h"
#include <qpe/calendar.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qwidgetstack.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qpainter.h>
#include <qlistbox.h>
#include <qpe/resource.h>
#include <qpe/config.h>

#include "../debug.h"

QualendarWeek::QualendarWeek(QWidget *parent, const char *name, 
			     QArray<QualendarDB *> dataBases, 
			     QPopupMenu *menu) 
  : QualendarViewEx(parent, name, dataBases, menu) {

  header = new QualendarWeekHeader(this);

  mainArea = new QualendarMainArea(this);
  mainArea->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,
				      QSizePolicy::Expanding));

  makeConnections();

  menu->clear();
}
QString QualendarWeek::viewName() const {
  return "Week";
}
QString QualendarWeek::trViewName() const {
  return tr("Week");
}
QPixmap QualendarWeek::icon() const {
  return Resource::loadPixmap("qualendar/week.png");
}

void QualendarWeek::myMouseEvent(QMouseEvent *e) {
  if (e->type() == QEvent::MouseButtonPress &&
      dateRects(3, 1).contains(e->pos()))
    ;
  else
    QualendarViewEx::myMouseEvent(e);
}
QDate QualendarWeek::dateStart() const {
  int i = date_.dayOfWeek(); // Monday = 1, Sunday = 7
  if (QualendarGlobal::onMonday)
    return date_.addDays(-(i-1)); 
  else {
    if (i == 7)
      return date_;
    else
      return date_.addDays(-i);
  }
}

QDate QualendarWeek::dateEnd() const {
  return dateStart().addDays(6);
}

void QualendarWeek::loadConfig() {
  header->setFontSize(QualendarGlobal::fontSize);

  Config config("Qualendar");
  config.setGroup("Week");
  fontSize = config.readNumEntry("FontSize", QualendarGlobal::fontSize);
  if (fontSize < 0) 
    fontSize = font().pixelSize();
  dateFontSize = config.readNumEntry("DateFontSize", fontSize);
  todoStart = config.readBoolEntry("TodoStart", true);
  todoDue = config.readBoolEntry("TodoDue", true);

  eventsFirst = config.readBoolEntry("EventsFirst", false);
}

void QualendarWeek::setupDateRects() {
  QDate dStart = dateStart();
  dateRects.resize(4, 2);

  for (uint i=0; i<7; i++)
    dateRects[i].setDate(dStart.addDays(i));
}

void QualendarWeek::relayout() {
  DRAW_DEBUG(qDebug("QualendarWeek::relayout"));
#ifdef DEBUG_PROFILE
  QTime tm;
  tm.start();
#endif
  QWidget *w = mainArea;
  if (fontSize >= 0)
    w->setFont(QFont(w->font().family(), fontSize));
  QPainter p(w);
  QFontMetrics fm = p.fontMetrics();

  cellWidth = w->width()/2;
  cellHeight = w->height()/4;
  cellHeaderHeight = dateFontSize;
  headerHeight = 0;

  downPixmap = Resource::loadPixmap("qualendar/down").convertToImage()
    .smoothScale(fm.height(), fm.height()/2);


  // layout day cells
  for ( int l = 0; l < 2; ++l ) 
    for ( int k = 0; k < 4; ++k ) 
      dateRects(k, l).setRect(l*cellWidth, k*cellHeight,
			      cellWidth, cellHeight);

  itemRects.clear();

  for(int l = 0; l < 2; l++) {
    for(int k = 0; k < 4; k++) {
      if (l == 1 && k == 3)
	break;
      DateRect &dr = dateRects(k, l);
      int ix = 0, iy = cellHeaderHeight;

      if (eventsFirst) {
	iy = layoutEvents(dr, fm, ix, iy);
	layoutTasks(dr, fm, ix, iy);
      } else {
	iy = layoutTasks(dr, fm, ix, iy);
	layoutEvents(dr, fm, ix, iy);
      }
    }
  }

#ifdef DEBUG_PROFILE
  qDebug("QualendarWeek::relayout %d ms", tm.elapsed());
#endif
}

void QualendarWeek::draw(QRect, bool useDirty) {
#ifdef DEBUG_PROFILE
  QTime tm;
  tm.start();
#endif
  QWidget *w = mainArea;
  QPainter p(w);
  QFontMetrics fm = p.fontMetrics();

  bool onMonday = QualendarGlobal::onMonday;

  // background
  if (!useDirty) {
    p.fillRect(0, cellHeight * 4 + 1, 
	       w->width(), w->height() - cellHeight * 4, 
	       QColor(255, 255, 255));
  }
  QRect updatedRect;
  for (int l = 0; l < 2; l++) {
    for (int k = 0; k < 4; k++) {
      const DateRect &dr = dateRects(k, l);
      if (useDirty && !dr.dirty())
	continue;
      DRAW_DEBUG(qDebug("Draw background %d %d", k, l));
      QColor c;
      c = getDayColor(dr.date());
      if (selectionMode &&
	  dr.date() >= selStart && dr.date() <= selEnd)
	p.fillRect(dr, QColor(0, 0, 255));
      else {
	p.fillRect(dr, c);
	if (!dr.background().isNull())
	  p.drawPixmap(dr.x() + dr.width() - dr.background().width() - 1, 
		       dr.y() + cellHeaderHeight, 
		       dr.background());
      }
      updatedRect |= dr;
    }
  }

  // grid
  p.drawLine(0, 0, w->width(), 0);
  for ( int y = 0; y <= 4; ++y )
    p.drawLine(0, headerHeight + y*cellHeight,
			   w->width(), headerHeight + y*cellHeight);
  for ( int x = 0; x <= 2; ++x )
    p.drawLine(x*cellWidth, 0, x*cellWidth, w->height());

  // event description
  QRect updatedRect1 = updatedRect;
  {
    p.setClipping(true);
    p.setClipRect(0, 0, w->width(), w->height());
    for(uint i = 0; i < itemRects.count(); i++) {
      ItemRect er = itemRects[i];
      if (er.intersects(updatedRect)) {
	DRAW_DEBUG(qDebug("Draw event %d", i));
	er.draw(p);
	updatedRect1 |= er;
      }
    }
    p.setClipping(false);
  }

  // day number

  for ( int l = 0; l < 2; l++ ) {
    for ( int k = 0; k < 4; k++ ) {
      if (l == 1 && k == 3)
	break;
      const DateRect &dr = dateRects(k, l);
      if (useDirty && !dr.intersects(updatedRect1))
	continue;
      int dy = dr.date().day();

      QDate d = dr.date();
      QString holDesc;
      p.save();
      p.setFont(QFont(p.font().family(), dateFontSize));
      p.fillRect(dr.x() + 1, dr.y() + 1, dr.width() - 1, fm.height() - 1, 
		 QColor(160, 220, 160));
      if (QualendarGlobal::holidays.contains(d)) {
	p.setPen(QColor(255, 0, 0));
	holDesc = QualendarGlobal::holidays[d];
      } else if ((d.dayOfWeek())%7 == 0) 
	p.setPen(QColor(255, 0, 0));
      else if ((d.dayOfWeek()) == 6) 
	p.setPen(QColor(0, 0, 255));
      else
	p.setPen(QColor(0, 0, 0));
      p.drawText(dr.x() + 3, dr.y() + fm.height(), 
		 QString::number(d.month()) + "/" +
		 QString::number(d.day()) + "(" +
		 dayName(d.dayOfWeek()%7) + ") " +
		 holDesc);
      p.restore();

      if (dr.tooMany) 
	p.drawPixmap(dr.x() + (dr.width() - downPixmap.width())/2, 
		     dr.y() + dr.height() - downPixmap.height(), 
		     downPixmap);
    }
  }

  DRAW_DEBUG(qDebug("highlightDate " + viewName() + "%d %d %d", date_.year(), date_.month(), date_.day()));
  p.setPen(QColor(0, 0, 0));
  int i, j;
  if (findDate(date_, i, j)) {
    QRect r = dateRects(i, j);
    p.drawRect(r.x()+1, r.y()+1, r.width()-1, r.height()-1);
  }

  for(int l = 0; l < 2; l++) 
    for(int k = 0; k < 4; k++) 
      dateRects(k, l).setDirty(false);

  // Todo cell.
  QRect dr = dateRects(3, 1);
  p.save();
  p.setFont(QFont(p.font().family(), dateFontSize));
  if (!useDirty)
    p.fillRect(dr.x() + 1, dr.y() + 1, dr.width() - 1, fm.height() - 1, 
	       QColor(160, 128, 128));
  p.fillRect(dr.x() + 1, dr.y() + fm.height(), 
	     dr.width()-2, dr.height() - fm.height() - 1, 
	     QColor(255, 255, 255));
  p.setPen(QColor(0, 0, 0));
  p.drawText(dr.x() + 1, dr.y() + fm.height() - 1, "Todo");
  p.restore();
  
  // tasks.
  QValueList<QualendarTask> tasks = getTasks(date_, date_, filter_);
  int x = dr.x() + 1, 
    y = dr.y() + fm.height() + cellHeaderHeight;
  p.setClipping(true);
  p.setClipRect(dr);
  for(uint i=0; i < tasks.count(); i++) {
    p.drawText(x, y, 
	       dateString(tasks[i], true, true) 
	       + " " + tasks[i].description());
    y += fm.height();
  }
  p.setClipping(false);

#ifdef DEBUG_PROFILE
  qDebug("QualendarWeek::draw, useDirty=%d, %d ms", useDirty, tm.elapsed());
#endif
}

QualendarItem QualendarWeek::selectedItem() {
#if 0
  int k, l;
  if (findDate(date_, k, l)) 
    if (dateRects(k, l).events().count() > 0)
      return dateRects(k, l).events()[0].event();
#endif
  return QualendarItem();
}

void QualendarWeek::slotViewOption(QAction *a) {
  QString t = a->text();
  redraw();
}

int QualendarWeek::layoutEvents(DateRect &dr, const QFontMetrics &fm, 
				 int ix, int iy) {
  int ox = dr.x(), oy = dr.y();
  QDate d = dr.date();

  for(uint i = 0; i < dr.events().count(); i++) {
    if (iy >= cellHeight - fm.height()*2/3)
      dr.tooMany = true;

    ItemRect er;

    QDateTime dt;
    const EffectiveEvent &eev = dr.events()[i];
    const Event &ev = eev.event();

    QPixmap pm = QualendarGlobal::getBackground(ev.description(), cellWidth, cellHeight-cellHeaderHeight);
    if (!pm.isNull()) {
      dr.setBackground(pm);
      dr.setEventHidden(i, true);
      continue;
    } else
      dr.setEventHidden(i, false);

    if (iy >= cellHeight)
      continue;

    {
      er.addText(fm, timeString(eev) + " ", 
		 (ev.type() == Event::AllDay) ? 
		 QColor(0, 255, 0) : QColor(0, 0, 255));
      QColor color = getColor(ev);

      QString s = ev.description();
      QPixmap pm;

      s = getIcon(s, fontSize, pm);
      er.moveTopLeft(QPoint(ox+ix, oy+iy));
      if (!pm.isNull()) 	
	er.addPixmap(pm);
      er.addText(fm, s, color);

      if (!ev.location().isEmpty())
	er.addText(fm, QString(" (%1)").arg(ev.location()), 
		   color);

      iy += fm.height();
      er.setItem(eev);
      er.clip(dr);
      itemRects.append(er);
    }
  }
  return iy;
}

int QualendarWeek::layoutTasks(DateRect &dr, const QFontMetrics &fm, 
				 int ix, int iy) {
  int ox = dr.x(), oy = dr.y();
  QDate d = dr.date();

  for(uint i = 0; i < dr.tasks().count(); i++) {
    if (iy >= cellHeight - fm.height()*2/3)
      dr.tooMany = true;
    if (iy >= cellHeight) 
      continue;

    ItemRect er;

    const QualendarTask &task = dr.tasks()[i];
    QColor color = getTodoColor(task, dr.date());
    QString s = task.description();
    QPixmap pm;

    s = getIcon(s, fontSize, pm);
    er.moveTopLeft(QPoint(ox+ix, oy+iy));
    er.addPixmap(getTodoPixmap(task, dr.date(), fontSize));
    if (!pm.isNull()) 	
      er.addPixmap(pm);
    QString t = QString(" %1 ").arg(task.priority());
    QString u = dateString(task, true, true);
    if (!u.isEmpty())
      t += "(" + u + ") ";
    er.addText(fm, t, color);

    er.addText(fm, s, color);
    iy += fm.height();
    er.setItem(task);
    er.clip(dr);
    itemRects.append(er);
  }
  return iy;
}
