#include "qualendaryear.h"
#include "../qualendarglobal.h"
#include "qualendarviewutil.h"

#include <qframe.h>
#include <qpopupmenu.h>
#include <qlayout.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qpainter.h>
#include <qtoolbutton.h>

#include <qpe/calendar.h>
#include <qpe/resource.h>
#include <qpe/config.h>

#include "../debug.h"

QualendarYear::QualendarYear(QWidget *parent, const char *name, 
			     QArray<QualendarDB *> dataBases, 
			     QPopupMenu *menu) 
  : QualendarView(parent, name, dataBases, menu), menu_(menu) {
  QFrame *f = new QFrame(this);
  f->setFixedHeight(QualendarGlobal::fontSize * 2);

  QHBoxLayout *l = new QHBoxLayout(f);

  sbYear = new QSpinBox(1970, 2037, 1, f);
  sbYear->setFocusPolicy(NoFocus);
  l->addWidget(sbYear);
  connect(sbYear, SIGNAL(valueChanged(int)), this, SLOT(slotSpinYear()));

  QToolButton *tb;

  tb = new QToolButton(f);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("back"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotB()));

  tb = new QToolButton(f);
  tb->setFocusPolicy(NoFocus);
  tb->setPixmap(Resource::loadPixmap("forward"));
  tb->setAutoRaise(true);
  l->addWidget(tb);
  connect(tb, SIGNAL(pressed()), this, SLOT(slotF()));

  mainArea = new QualendarMainArea(this);
  connect(mainArea, SIGNAL(paintMe(QRect)), this, SLOT(paintMainArea()));
  connect(mainArea, SIGNAL(mouseEvent(QMouseEvent *)), 
	  this, SLOT(mainAreaMouseEvent(QMouseEvent *)));
  mainArea->setFocusPolicy(NoFocus);


  menu->clear();

  ignoreSignal = true;
}

QString QualendarYear::viewName() const {
  return "Year";
}
QString QualendarYear::trViewName() const {
  return tr("Year");
}
QPixmap QualendarYear::icon() const {
  return Resource::loadPixmap("qualendar/year.png");
}
QDate QualendarYear::currentDate() const {
  return date_;
}
void QualendarYear::selectedDateTime(QDateTime &start, QDateTime &end) const {
  start = QDateTime(date_, QTime(10, 0, 0));
  end = QDateTime(date_, QTime(12, 0, 0));
}
void QualendarYear::setupMenu() {
}

void QualendarYear::resizeEvent(QResizeEvent *e) {
  int dw = mainArea->width() / 33;
  if (dw >= 14) 
    wide = true;
  else
    wide = false;
}

void QualendarYear::keyPressEvent(QKeyEvent *e) {
  switch(e->key()) {
  case Key_Return:
    e->accept();
    break;
  case Key_Left:
    slotB();
    break;
  case Key_Right:
    slotF();
    break;
  default:
    QualendarView::keyPressEvent(e);
  }
}

void QualendarYear::setFilter(QMap<int, bool>, bool) {
}

void QualendarYear::setDate(const QDate &d, bool mustDraw) {
  QDate oldDate = date_;
  date_ = d;
  if (oldDate.year() != date_.year()) {
    ignoreSignal = true;
    sbYear->setValue(date_.year());
    ignoreSignal = false;
  }

  if (mustDraw)
    mainArea->update();
  if (mustDraw) {
    setHighlighted(oldDate, false);
    setHighlighted(date_, true);
  }
}

void QualendarYear::loadConfig() {
  sbYear->setFixedSize(QualendarGlobal::fontSize * 5, 
		       QualendarGlobal::fontSize * 2);
}

void QualendarYear::redraw() {
  mainArea->update();
}

void QualendarYear::slotSpinYear() {
  if (ignoreSignal)
    return;
  QDate d(sbYear->value(), date_.month(), 
	  QMIN(QDate(sbYear->value(), date_.month(), 1).daysInMonth(), 
	       date_.day()));
  sbYear->setEnabled(false);
  setDate(d);
  sbYear->setEnabled(true);
}

void QualendarYear::slotB() {
  int y = date_.year(), m = date_.month(), d = date_.day();
  if (wide) {
    y--;
  } else {
    m -= 6;
    if (m <= 0) {
      m += 12;
      y--;
    }
  }
  setDate(QDate(y, m, QMIN(d, QDate(y, m, 1).daysInMonth())));
}
void QualendarYear::slotF() {
  int y = date_.year(), m = date_.month(), d = date_.day();
  if (wide) {
    y++;
  } else {
    m += 6;
    if (m > 12) {
      m -= 12;
      y++;
    }
  }
  setDate(QDate(y, m, QMIN(d, QDate(y, m, 1).daysInMonth())));
}

// horizontal margin and spacing = width of day.
// margin, (days * 7, spacing)*3 days * 7, rest >= margin)
// vertical margin and spacing = height of day.
// margin, (name, header, days * 6, spacing)*2, (name, header, days * 6)
#define NCOLS (wide ? 4 : 2)
#define MONTHX(x) (((x-1)%NCOLS)*(dw*8) + dw)
#define MONTHY(x) (((x-1)%(wide ? 12 : 6))/NCOLS*(dh*8) + dh/2)

void QualendarYear::paintMainArea() {
  if (wide)
    dw = mainArea->width() / 33;
  else
    dw = mainArea->width() / 17;
  dh = mainArea->height() / 24;

  QPainter p(mainArea);

  p.fillRect(0, 0, mainArea->width(), mainArea->height(),
  	     QColor(255, 255, 192));

  int y = date_.year();
  bool onMonday = QualendarGlobal::onMonday;

  mainArea->setFont(font().family(), 10);

  /*
  QString s[7];
  s[0] = tr("S");
  s[1] = tr("M");
  s[2] = tr("T");
  s[3] = tr("W");
  s[4] = tr("Th");
  s[5] = tr("F");
  s[6] = tr("Sa");
  */

  uint ht = p.fontMetrics().height();
  uint mOff = (wide ? 0 : (date_.month() <= 6 ? 0 : 6)),
    nMonth = (wide ? 12 : 6);
  for(int j = 1; j <= nMonth; j++) {
    p.fillRect(MONTHX(j)-dw/4, MONTHY(j)-dh/4, dw * 15/2, dh * 15/2, 
	       QColor(255, 255, 255));
    p.fillRect(MONTHX(j)-dw/4, MONTHY(j)-dh/4, dw * 15/2, dh * 5/4, 
	       QColor(192, 192, 255));
    p.setPen(QColor(128, 128, 128));
    p.drawRect(MONTHX(j)-dw/4, MONTHY(j)-dh/4, dw * 15/2, dh * 15/2);
    p.setPen(QColor(0, 0, 0));
    p.drawText(MONTHX(j), MONTHY(j) - dh/8 + ht - 2, 
	       QObject::tr(QDate(y, j+mOff, 1).monthName(j+mOff)));
    /*
    for(uint i = 0; i < 7; i++) {
      switch ((i + (onMonday ? 1 : 0)) % 7) {
      case 0:
	p.setPen(QColor(255, 0, 0));
	break;
      case 6:
	p.setPen(QColor(0, 0, 255));
	break;
      default:
	p.setPen(QColor(0, 0, 0));
	break;
      }
      p.drawText(MONTHX(m) + dw*i, MONTHY(m) + ht, 
		 s[(i + (onMonday ? 1 : 0)) % 7]);
    }
    */

    p.save();
    p.setFont(QFont(font().family(), 10));
    uint x0 = (QDate(y, j+mOff, 1).dayOfWeek() - (onMonday ? 1 : 0)) % 7, 
      y0 = 1;
    for(uint i = 1; i <= QDate(y, j+mOff, 1).daysInMonth(); i++) {
      if (QDate(y, j+mOff, i) == QDate::currentDate()) {
	p.fillRect(MONTHX(j) + x0 * dw - 2, MONTHY(j) + y0 * dh + 2,
		   14, 13, QColor(255, 255, 0));
      } 
      if (QDate(y, j+mOff, i) == date_) {
	p.setPen(QColor(0, 0, 0));
	p.drawRect(MONTHX(j) + x0 * dw - 2, MONTHY(j) + y0 * dh + 2, 
		   14, 13);
      } 
      switch ((x0 + (onMonday ? 1 : 0)) % 7) {
      case 0:
	p.setPen(QColor(255, 0, 0));
	break;
      case 6:
	p.setPen(QColor(0, 0, 255));
	break;
      default:
	p.setPen(QColor(0, 0, 0));
	break;
      }
      if (QualendarGlobal::holidays.contains(QDate(y, j+mOff, i)))
	p.setPen(QColor(255, 0, 0));
      p.drawText(MONTHX(j) + x0 * dw, MONTHY(j) + y0 * dh + ht, 
		 QString("%1").arg(i, 2));
      x0 ++;
      if (x0 == 7) {
	x0 = 0;
	y0 ++;
      }
    }
    p.restore();
  }
}

void QualendarYear::mainAreaMouseEvent(QMouseEvent *) {
}

void QualendarYear::setHighlighted(QDate, bool) {
}
